/***************************************************************************
                          page.cpp  -  description
                             -------------------
    begin                : Sat Apr 7 2001
    copyright            : (C) 2001 by Franz Schmid
    email                : Franz.Schmid@altmuehlnet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "page.h"
#include "pageitem.h"
#include <qapplication.h>
#include <qcolor.h>
#include <qrect.h>
#include <qpointarray.h>
#include <qregion.h>
#include <qcursor.h>
#include <qimage.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qcstring.h>
#include <math.h>
#include <stdio.h>
#include <qbitmap.h>
#include <qpopupmenu.h>
// #include <iostream.h>

static const char *Rotieren[] = {
"16 14 3 1",
". c None",
"a c #000000",
"# c #7b7b7b",
".....##aaa#.....",
"....#aaaaaaa....",
"...#aa#....#a...",
"...aa#..........",
"..aaa.......a...",
"aaaaaaa....aaa..",
".aaaaa....aaaaa.",
"..aaa....aaaaaaa",
"...a.......aaa..",
"..........#aa...",
"...a#....#aa#...",
"....aaaaaaa#....",
".....#aaa##.....",
"................"};

static const char *HandC[]={
"16 16 2 1",
". c None",
"# c #000000",
".......##.......",
"...##.#..###....",
"..#..##..#..#...",
"..#..##..#..#.#.",
"...#..#..#..##.#",
"...#..#..#..#..#",
".##.#.......#..#",
"#..##..........#",
"#...#.........#.",
".#............#.",
"..#...........#.",
"..#..........#..",
"...#.........#..",
"....#.......#...",
".....#......#...",
".....#......#..."};

Page::Page(QWidget *pa, int x, int y, int b, int h, ScribusDoc *doc, QScrollView *view) : QWidget(pa)
{
	resize(b, h);
	move(x, y);
	setBackgroundColor(QColor(255,255,255));
	doku = doc;
	Anz = view;
	Items.clear();
	Items.setAutoDelete(true);
	Margins.Top = 9;
	Margins.Left = 9;
	Margins.Right = 9;
	Margins.Bottom = 40;
	SelItem.clear();
	ClRe = -1;
	emit DocChanged();
}
Page::~Page(){
}
/** No descriptions */
void Page::paintEvent(QPaintEvent *e)
{
	QPainter p;
	uint a;
	int b;
	p.begin(this);
	p.setPen(QColor(0, 0, 0));
	p.drawRect(0, 0, width(), height());
	if (doku->Marks)
		{
		p.setPen(blue);
		p.drawLine(0, int(Margins.Top*doku->Scale), width(), int(Margins.Top*doku->Scale));
		p.drawLine(0, height()-int(Margins.Bottom*doku->Scale), width(), height()-int(Margins.Bottom*doku->Scale));
		p.drawLine(int(Margins.Left*doku->Scale), 0, int(Margins.Left*doku->Scale), height());
		p.drawLine(width()-int(Margins.Right*doku->Scale), 0, width()-int(Margins.Right*doku->Scale), height());
		}
	if (doku->Raster)
		{
		p.setClipRect(e->rect());
		int stx = e->rect().x();
		int endx = e->rect().x()+e->rect().width();
		int sty = e->rect().y();
		int endy = e->rect().y()+e->rect().height();
		if (doku->Scale > 0.49)
			{
			for (b = sty; b < endy; b++)
				{
				if ((b % qRound(doku->minorGrid*doku->Scale)) == 0)
					{
					p.setPen(QPen(doku->minorColor, 1, DotLine, FlatCap, MiterJoin));
					p.drawLine(0, b, width(), b);
					}
				if ((b % qRound(doku->majorGrid*doku->Scale)) == 0)
					{
					p.setPen(QPen(doku->majorColor, 1, SolidLine, FlatCap, MiterJoin));
					p.drawLine(0, b, width(), b);
					}
				}
			for (b = stx; b < endx; b++)
				{
				if ((b % qRound(doku->minorGrid*doku->Scale)) == 0)
					{
					p.setPen(QPen(doku->minorColor, 1, DotLine, FlatCap, MiterJoin));
					p.drawLine(b, 0, b, height());
					}
				if ((b % qRound(doku->majorGrid*doku->Scale)) == 0)
					{
					p.setPen(QPen(doku->majorColor, 1, SolidLine, FlatCap, MiterJoin));
					p.drawLine(b, 0, b, height());
					}
				}
			}
		}
	p.end();
	if (Items.count() != 0)
		{
		for (a = 0; a < Items.count(); a++)
			{
			QPainter p;
			p.begin(this);
			Transform(Items.at(a), &p);
//			QRegion apr = QRegion(p.xForm(Items.at(a)->Clip));
			QRegion apr = QRegion(p.xForm(QPointArray(QRect(0, 0, int(Items.at(a)->Width), int(Items.at(a)->Height)))));
			p.end();
			if (e->rect().intersects(apr.boundingRect()))
				{
				Items.at(a)->paintObj(e->rect());
				if ((doku->EditClip) && (Items.at(a)->Select))
					{
					MarkClip(Items.at(a));
					}
				if ((doku->AppMode == 7) && (Items.at(a)->Select))
					{
					slotDoCurs(true);
					}
				}
			}
		}
}

void Page::Transform(PageItem *b, QPainter *p)
{
	p->translate(int(b->Xpos*doku->Scale), int(b->Ypos*doku->Scale));
	p->scale(doku->Scale, doku->Scale);
	p->rotate(b->Rot);
}

void Page::TransformM(PageItem *b, QPainter *p)
{
	if (b->flippedH % 2 != 0)
		{
		p->translate(b->Width, 0);
		p->scale(-1, 1);
		}
	if (b->flippedV % 2 != 0)
		{
		p->translate(0, b->Height);
		p->scale(1, -1);
		}
}

bool Page::MoveSizeItem(QPoint newX, QPoint newY, int ite)
{
	QRegion alt, neu;
	QPainter p;
	PageItem *b = Items.at(ite);
	p.begin(this);
	Transform(b, &p);
	alt = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height)));
	p.end();
	b->Width -= newY.x();
	b->Height -= newY.y();
	b->Xpos += newX.x();
	b->Ypos += newX.y();
	switch (b->PType)
		{
		case 1:
			b->Clip.makeEllipse(0, 0, int(b->Width), int(b->Height));
			break;
		case 5:
			b->Clip.setPoints(4, -1,-1, int(b->Width+1),-1, int(b->Width+1),int(b->Height+1), -1,int(b->Height+1));
			break;
		default:
			b->Clip.setPoints(4, 0,0, int(b->Width),0, int(b->Width),int(b->Height), 0,int(b->Height));
			break;
		}
	p.begin(this);
	Transform(b, &p);
	neu = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height))).unite(alt);
	p.end();
	QRect g = neu.boundingRect();
	g.setX(g.x()-20);
	g.setY(g.y()-20);
	g.setWidth(g.width()+20);
	g.setHeight(g.height()+20);
	repaint(g);
	emit ItemPos(b->Xpos, b->Ypos);
	emit ItemGeom(b->Width, b->Height);
	return TRUE;	
}

bool Page::SizeItem(float newX, float newY, int ite)
{
	QRegion alt, neu;
	QPainter p;
	PageItem *b = Items.at(ite);
	if (b->PType != 5)
		{
		if ((newX < 2) || (newY < 2)) { return FALSE; }
		}
	p.begin(this);
	Transform(b, &p);
	alt = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height)));
	p.end();
	b->Width = newX;
	b->Height = newY;
	switch (b->PType)
		{
		case 1:
			b->Clip.makeEllipse(0, 0, int(b->Width), int(b->Height));
			break;
		case 5:
			b->Clip.setPoints(4, -1,-1, int(b->Width+1),-1, int(b->Width+1),int(b->Height+1), -1,int(b->Height+1));
			break;
		default:
			b->Clip.setPoints(4, 0,0, int(b->Width),0, int(b->Width),int(b->Height), 0,int(b->Height));
			break;
		}
	p.begin(this);
	Transform(b, &p);
	neu = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height))).unite(alt);
	p.end();
	QRect g = neu.boundingRect();
	g.setX(g.x()-20);
	g.setY(g.y()-20);
	g.setWidth(g.width()+20);
	g.setHeight(g.height()+20);
	repaint(g);
	emit ItemGeom(b->Width, b->Height);
	return TRUE;	
}

void Page::MoveItemI(float newX, float newY, int ite)
{
	QPainter p;
	PageItem *b = Items.at(ite);
	p.begin(this);
	Transform(b, &p);
	TransformM(b, &p);
	QRegion alt = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height)));
	p.end();
	if (b->flippedH % 2 != 0)
		{
		b->LocalX -= newX;
		}
	else
		{
		b->LocalX += newX;
		}
	if (b->flippedV % 2 != 0)
		{
		b->LocalY -= newY;
		}
	else
		{
		b->LocalY += newY;
		}
	repaint(alt);
	emit SetLocalValues(b->LocalScX, b->LocalScY, b->LocalX, b->LocalY);	
}

bool Page::MoveItem(float newX, float newY, int ite)
{
	QRegion alt, neu;
	QPainter p;
	PageItem *b;
	float oldx, oldy;
	bool retw = false;
	b = Items.at(ite);
	p.begin(this);
	Transform(b, &p);
	alt = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height)));
	p.end();
	oldx = b->Xpos;
	oldy = b->Ypos;
	b->Xpos += newX;
	b->Ypos += newY;
	if (doku->useRaster)
		{
		b->Xpos = qRound(b->Xpos / doku->minorGrid) * doku->minorGrid;
		b->Ypos = qRound(b->Ypos / doku->minorGrid) * doku->minorGrid;
		}
	if ((oldx != b->Xpos) || (oldy != b->Ypos))
		{
		retw = true;
		}
	p.begin(this);
	Transform(b, &p);
	neu = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height))).unite(alt);
	p.end();
	QRect g = neu.boundingRect();
	g.setX(g.x()-20);
	g.setY(g.y()-20);
	g.setWidth(g.width()+20);
	g.setHeight(g.height()+20);
	repaint(g);
	emit ItemPos(b->Xpos, b->Ypos);
	return retw;	
}

float Page::xy2Deg(float x, float y)
{
	float a = asin ( y / sqrt ( x * x + y * y ) );
	if ((a>0) && (x>0)) { return a * 180 / 3.1415927; }
	if ((a>0) && (x<0)) { return 180 - a * 180 / 3.1415927; }
	if ((a<0) && (x<0)) { return 180 +  ( -a * 180 / 3.1415927 ); }
	if ((a<0) && (x>0)) { return 360 + a * 180 / 3.1415927; }
	if ((a<0) && (x==0)) { return 270.0; }
	if ((a>0) && (x==0)) { return 90.0; }
	if ((a==0) && (x<0)) { return 180.0; }
	if ((a==0) && (x>0)) { return 0.0; }
	return 0.0;
}

void Page::RotateItem(float win, int ite)
{
	QRegion alt, neu;
	QPainter p;
	PageItem *b;
	b = Items.at(ite);
	p.begin(this);
	Transform(b, &p);
	alt = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height)));
	p.end();
	b->Rot = win;
	p.begin(this);
	Transform(b, &p);
	neu = QRegion(p.xForm(QRect(0, 0, b->Width, b->Height))).unite(alt);
	p.end();
	QRect g = neu.boundingRect();
	g.setX(g.x()-20);
	g.setY(g.y()-20);
	g.setWidth(g.width()+20);
	g.setHeight(g.height()+20);
	repaint(g);
	emit SetAngle(b->Rot);
}

void Page::mouseReleaseEvent(QMouseEvent *)
{
	Mpressed = false;
	mCG = false;
	if ((doku->AppMode != 6) && (doku->AppMode != 7))
		{
		doku->AppMode = 1;
		qApp->setOverrideCursor(QCursor(ArrowCursor), 1);
		emit PaintingDone();
		repaint();
		}
	if (doku->AppMode == 7)
		{
		PageItem *b = Items.at(*SelItem.at(0));
		uint a;
		if (b->PType == 4)
			{
			if (oldCp == b->CPos)
				{
				for (a = 0; a < b->Ptext.count(); a++)
					{
					b->Ptext.at(a)->cselect = false;
					}
				b->Dirty = true;
				b->paintObj();
				}
			}
		}
}

void Page::mouseMoveEvent(QMouseEvent *m)
{
	int newX, newY, dx, dy, c;
	uint a;
	PageItem *b;
	QPoint np, np2;
	QPainter p;
	QRect tx;
	bool erf = false;
	float sc = doku->Scale;
	emit Hrule(m->x());
	emit Vrule(m->y()+Anz->childY(parentWidget()));
	if (SelItem.count() != 0)
		{
		newX = int(m->x()/sc);
		newY = int(m->y()/sc);
		if (Mpressed && (doku->AppMode == 9))
			{
			float newW =  xy2Deg(newX - Items.at(*SelItem.at(0))->Xpos, newY - Items.at(*SelItem.at(0))->Xpos);
			RotateItem(qRound((Items.at(*SelItem.at(0))->Rot - (oldW - newW))*100.0)/100.0, *SelItem.at(0));
			oldW = newW;
			emit DocChanged();
			}
		if (Mpressed && (doku->AppMode == 7))
			{
			b = Items.at(*SelItem.at(0));
			if (b->PType == 2)
				{
				MoveItemI(newX-Mxp, newY-Myp, *SelItem.at(0));
				emit DocChanged();
				Mxp = newX;
				Myp = newY;
				}
			if (b->PType == 4)
				{
				for (a = 0; a < b->Ptext.count(); a++)
					{
					b->Ptext.at(a)->cselect = false;
					}
				b->HasSel = false;
				slotSetCurs(newX, newY);
				if (b->CPos < oldCp)
					{
					for (c = b->CPos; c < oldCp; c++)
						{
						b->Ptext.at(c)->cselect = true;
						b->HasSel = true;
						}
					}
				if (b->CPos > oldCp)
					{
					for (c = oldCp; c < b->CPos; c++)
						{
						b->Ptext.at(c)->cselect = true;
						b->HasSel = true;
						}
					}
				b->Dirty = true;
				b->paintObj();
				if (b->HasSel)
					{
					emit HasTextSel();
					}
				else
					{
					emit HasNoTextSel();
					}
				}
			}
		if (Mpressed && (doku->AppMode == 1))
			{
			if (doku->EditClip)
				{
				b = Items.at(*SelItem.at(0));
				p.begin(this);
				Transform(b, &p);
				TransformM(b, &p);
				np = p.xFormDev(m->pos());
				if (np.x() < -1) { np.setX(-1); }
				if (np.y() < -1) { np.setY(-1); }
				if (np.x() > b->Width+1) { np.setX(b->Width+1); }
				if (np.y() > b->Height+1) { np.setY(b->Height+1); }
				if (ClRe != -1)
					{
					b->Clip.setPoint(ClRe, np);
					repaint(QRegion(p.xForm(QRect(-5, -5, b->Width+10, b->Height+10))));
					}
				p.end();
				return;
				}
			if (mCG)
				{
				for (a = 0; a < SelItem.count(); a++)
					{
					switch (HowTo)
						{
						case 1:
							b = Items.at(*SelItem.at(a));
							p.begin(this);
							Transform(b, &p);
							np = p.xFormDev(QPoint(m->x(), m->y()));
							p.end();
							if (doku->useRaster)
								{
								dx = int(b->Xpos - int (b->Xpos / doku->minorGrid) * doku->minorGrid);
								dy = int(b->Ypos - int (b->Ypos / doku->minorGrid) * doku->minorGrid);
								np.setX((np.x() / doku->minorGrid) * doku->minorGrid - dx);
								np.setY((np.y() / doku->minorGrid) * doku->minorGrid - dy);
								}
							erf = SizeItem(np.x(), np.y(), *SelItem.at(a));
							break;
						case 2:
							b = Items.at(*SelItem.at(a));
							p.begin(this);
							p.rotate(b->Rot);
							np = p.xFormDev(QPoint(newX-Mxp, newY-Myp));
							p.end();
							np2 = QPoint(newX-Mxp, newY-Myp);
							erf = MoveSizeItem(np2, np, *SelItem.at(a));
							break;
						}
					}
				if (erf)
					{
					Mxp = newX;
					Myp = newY;
					emit DocChanged();
					}
				}
			else
				{
				for (a = 0; a < SelItem.count(); a++)
					{
					b = Items.at(*SelItem.at(a));
					erf = MoveItem(newX-Mxp, newY-Myp, *SelItem.at(a));
					}
				if (erf)
					{
					emit DocChanged();
					Mxp = newX;
					Myp = newY;
					}
				}
			}
		if (!Mpressed)
			{
			for (a = 0; a < SelItem.count(); a++)
				{
  			b = Items.at(*SelItem.at(a));
  			p.begin(this);
				Transform(b, &p);
				if (QRegion(p.xForm(QPointArray(QRect(-3, -3, int(b->Width+6), int(b->Height+6))))).contains(m->pos()))
  				{
  				tx = p.xForm(QRect(b->Width-6, b->Height-6, 6, 6));
  				if (tx.contains(m->pos()))
  					{
  					if (doku->AppMode == 9)
  						{
  						qApp->setOverrideCursor(QCursor(QPixmap(Rotieren)), 1);
  						}
  					else
  						{
  						if (doku->EditClip)
  							{
  							qApp->setOverrideCursor(QCursor(crossCursor), 1);
  							}
  						else
  							{
  							qApp->setOverrideCursor(QCursor(SizeFDiagCursor), 1);
  							}
  						}
  					}
  				else
  					{
  					tx = p.xForm(QRect(0, 0, 6, 6));
  					if (tx.contains(m->pos()))
  						{
  						if (doku->EditClip)
  							{
  							qApp->setOverrideCursor(QCursor(crossCursor), 1);
  							}
  						else
  							{
  							qApp->setOverrideCursor(QCursor(SizeFDiagCursor), 1);
  							}
  						}
  					else
  						{
  						if (doku->EditClip)
  							{
  							qApp->setOverrideCursor(QCursor(crossCursor), 1);
  							}
  						else
  							{
  							qApp->setOverrideCursor(QCursor(SizeAllCursor), 1);
  							if (doku->AppMode == 7)
  								{
  								if (b->PType == 4)
  									qApp->setOverrideCursor(QCursor(ibeamCursor), 1);
  								if (b->PType == 2)
  									qApp->setOverrideCursor(QCursor(QPixmap(HandC)), 1);
  								}
  							}
  						}
  					}
  				break;
  				}
  			else
  				{
  				qApp->setOverrideCursor(QCursor(ArrowCursor), 1);
  				}
  			p.end();
				}			
			}
		}
}

void Page::mousePressEvent(QMouseEvent *m)
{
	float sc;
	int z;
	int Rxp = 0;
	int Ryp = 0;
	int Rxpd = 0;
	int Rypd = 0;
	sc = doku->Scale;
	Mpressed = true;
	Mxp = int(m->x()/sc);
	Myp = int(m->y()/sc);
	if ((doku->useRaster) && (!doku->AppMode == 7))
		{
		Rxp = Mxp / doku->minorGrid * doku->minorGrid;
		Rxpd = Mxp - Rxp;
		Mxp = Rxp;
		Ryp = Myp / doku->minorGrid * doku->minorGrid;
		Rypd = Myp - Ryp;
		Myp = Ryp;
		}
	setFocus();
	switch (doku->AppMode)
		{
		case 1:
			SeleItem(m);
			if ((SelItem.count() != 0) && (m->button() == RightButton))
				{
				QPopupMenu *pmen = new QPopupMenu();
				qApp->setOverrideCursor(QCursor(ArrowCursor), 1);
				PageItem *b = Items.at(*SelItem.at(0));
				if (b->PType == 2)
					{
					int px = pmen->insertItem(tr("Image visible"), this, SLOT(TogglePic()));
					if (b->PicArt)
						{
						pmen->setItemChecked(px, true);
						}
					}
				if (b->PType == 4)
					{
					int pxb = pmen->insertItem(tr("Is PDF-Bookmark"), this, SLOT(ToggleBookmark()));
					if (b->isBookmark)
						{
						pmen->setItemChecked(pxb, true);
						}
					}
				pmen->insertItem(tr("Send to Back"), this, SLOT(ToBack()));
				pmen->insertItem(tr("Bring to Front"), this, SLOT(ToFront()));
				pmen->insertItem(tr("Lower"), this, SLOT(LowerItem()));
				pmen->insertItem(tr("Raise"), this, SLOT(RaiseItem()));
				pmen->insertSeparator();
				pmen->insertItem(tr("Cut"), this, SIGNAL(CutItem()));
				pmen->insertItem(tr("Copy"), this, SIGNAL(CopyItem()));
				pmen->insertItem(tr("Delete"), this, SLOT(DeleteItem()));
				pmen->exec(QCursor::pos());
				Mpressed = false;
				delete pmen;
				}			
			break;
		case 2:
			Deselect();
			z = PaintRect(Mxp, Myp, 1+Rxpd, 1+Rypd, doku->Dwidth, doku->Dbrush, doku->Dpen);
			SetupDraw(z);
			emit HaveSel(3);
			break;
		case 3:
			Deselect();
			z = PaintEllipse(Mxp, Myp, 1+Rxpd, 1+Rypd, doku->Dwidth, doku->Dbrush, doku->Dpen);
			SetupDraw(z);
			emit HaveSel(1);
			break;
		case 4:
			Deselect();
			z = PaintPict(Mxp, Myp, 1+Rxpd, 1+Rypd);
			SetupDraw(z);
			emit HaveSel(2);
			break;
		case 5:
			Deselect();
			z = PaintText(Mxp, Myp, 1+Rxpd, 1+Rypd, doku->Dwidth, doku->Dpen);
			SetupDraw(z);
			emit HaveSel(4);
			break;
		case 6:
			if (m->state() == ShiftButton)
				{
				emit ZoomOut();
				}
			else
				{
				emit ZoomIn();
				}
			break;
		case 7:
			slotDoCurs(false);
			slotSetCurs(Mxp, Myp);
			oldCp = Items.at(*SelItem.at(0))->CPos;
			slotDoCurs(true);
			break;
		case 8:
			Deselect();
			z = PaintLine(Mxp, Myp, 1+Rxpd, 1+Rypd, doku->Dwidth, doku->Dpen);
			SetupDraw(z);
			emit HaveSel(5);
			break;
		case 9:
			if (SelItem.count() != 0)
				{
				oldW = xy2Deg(Mxp - Items.at(*SelItem.at(0))->Xpos, Myp - Items.at(*SelItem.at(0))->Xpos);
				}
			break;
		}
}

bool Page::SeleItem(QMouseEvent *m)
{
	uint a;
	float sc = doku->Scale;
	QPainter p;
	QRect tx;
	PageItem* b = Items.last();
	Mpressed = true;
	Mxp = int(m->x()/sc);
	Myp = int(m->y()/sc);
	if ((doku->EditClip) && (SelItem.count() != 0))
		{
		int x, y;
		QPoint np;
		b = Items.at(*SelItem.at(0));
		ClRe = -1;
		p.begin(this);
		Transform(b, &p);
		TransformM(b, &p);
		for (a=0; a<b->Clip.size(); a++)
			{
			b->Clip.point(a, &x, &y);
			np = p.xForm(QPoint(x, y));
			tx = QRect(np.x()-3/sc, np.y()-3/sc, 6/sc, 6/sc);
			if (tx.contains(m->pos()))
				{
				if (m->state() == ShiftButton)
					{
					np = p.xFormDev(m->pos());
					QPointArray cli = QPointArray();
					uint cc, cc2;
					cc2 = 0;
					for (cc=0; cc<b->Clip.size(); cc++)
						{
						b->Clip.point(cc, &x, &y);
						cli.putPoints(cc2, 1, x, y);
						cc2++;
						if (cc == a)
							{
							cli.putPoints(cc2, 1, np.x(), np.y());
							cc2++;
							}
						}
					b->Clip = cli.copy();
					repaint(QRegion(p.xForm(QRect(-5, -5, b->Width+10, b->Height+10))));
					}
				if ((m->state() == ControlButton) && (b->Clip.size() > 3))
					{
					np = p.xFormDev(m->pos());
					QPointArray cli = QPointArray();
					uint cc, cc2;
					cc2 = 0;
					for (cc=0; cc<b->Clip.size(); cc++)
						{
						b->Clip.point(cc, &x, &y);
						cli.putPoints(cc2, 1, x, y);
						cc2++;
						if (cc == a)
							{
							cc2--;
							}
						}
					b->Clip = cli.copy();
					repaint(QRegion(p.xForm(QRect(-5, -5, b->Width+10, b->Height+10))));
					ClRe = -1;
					break;
					}
				ClRe = a;
				break;
				}
			}
		p.end();
		return true;
		}
	ClRe = -1;
	if ((m->state() != ShiftButton) || (doku->ActPage != this))
		{
		Deselect();
		doku->ActPage = this;
		emit PgCh(PageNr);
		}
	for (a = 0; a < Items.count(); a++)
		{
		p.begin(this);
		Transform(b, &p);
		if (QRegion(p.xForm(QPointArray(QRect(0, 0, int(b->Width), int(b->Height))))).contains(m->pos()))
			{
			if (!b->Select)
				{
				b->Select = true;
				b->paintObj();
				SelItem.append(&b->ItemNr);
				emit HaveSel(b->PType);
				emit ItemPos(b->Xpos, b->Ypos);
				emit ItemGeom(b->Width, b->Height);
				emit SetAngle(b->Rot);
				emit SetSizeValue(b->Pwidth);
				emit SetLineArt(b->PLineArt);
				emit SetLocalValues(b->LocalScX, b->LocalScY, b->LocalX, b->LocalY );
				emit ItemFarben(b->Pcolor2, b->Pcolor, b->Shade2, b->Shade);
				emit ItemTextAttr(b->LineSp);
				emit ItemTextFont(b->IFont);
				emit ItemTextSize(b->ISize);
				}
  		tx = p.xForm(QRect(b->Width-6, b->Height-6, 6, 6));
			if (tx.contains(m->pos()))
				{
				mCG = TRUE;
				HowTo = 1;
				qApp->setOverrideCursor(QCursor(SizeFDiagCursor), 1);
				}
			else
				{
				tx = p.xForm(QRect(0, 0, 6, 6));
				if (tx.contains(m->pos()))
					{
					mCG = TRUE;
					HowTo = 2;
					qApp->setOverrideCursor(QCursor(SizeFDiagCursor), 1);
					}
				else
					{
					mCG = FALSE;
					qApp->setOverrideCursor(QCursor(SizeAllCursor), 1);
					}
				}
			p.end();
			return TRUE;
			}
		p.end();
		b = Items.prev();
		}
	SelItem.clear();
	emit HaveSel(-1);
	return FALSE;
}

void Page::SelectItemNr(int nr)
{
	PageItem *b = Items.at(nr);
	b->Select = true;
	b->paintObj();
	SelItem.append(&b->ItemNr);
	emit HaveSel(b->PType);
	emit ItemPos(b->Xpos, b->Ypos);
	emit ItemGeom(b->Width, b->Height);
	emit SetAngle(b->Rot);
	emit SetSizeValue(b->Pwidth);
	emit SetLineArt(b->PLineArt);
	emit SetLocalValues(b->LocalScX, b->LocalScY, b->LocalX, b->LocalY );
	emit ItemFarben(b->Pcolor2, b->Pcolor, b->Shade2, b->Shade);
	emit ItemTextAttr(b->LineSp);
	emit ItemTextFont(b->IFont);
	emit ItemTextSize(b->ISize);
}

void Page::slotSetCurs(int x, int y)
{
  if (SelItem.count() != 0)
  	{
  	PageItem *b = Items.at(*SelItem.at(0));
  	if ((b->PType != 4) || (b->Ptext.count() == 0)) { return; }
  	QPainter p;
  	QFont ffo;
  	p.begin(this);
  	Transform(b, &p);
		TransformM(b, &p);
  	uint a;
  	int xp, yp, w, h;
  	for (a=0; a<b->Ptext.count(); a++)
  		{
  		xp = int(b->Ptext.at(a)->xp);
  		yp = int(b->Ptext.at(a)->yp);
  		ffo = doku->UsedFonts[b->Ptext.at(a)->cfont];
  		ffo.setPointSize(b->Ptext.at(a)->csize);
  		p.setFont(ffo);
  		w = p.fontMetrics().width(QChar(b->Ptext.at(a)->ch));
  		h = int(b->LineSp);
  		if (QRegion(p.xForm(QRect(xp-1, yp-h, w+1, h))).contains(QPoint(x, y)))
  			{
  			b->CPos = a;
  			p.end();
				emit ItemTextFont(b->Ptext.at(a)->cfont);
				emit ItemTextSize(b->Ptext.at(a)->csize);
  			return;
  			}
  		}
  	QPoint np;
  	for (a=0; a<b->Ptext.count(); a++)
  		{
  		xp = int(b->Ptext.at(a)->xp);
  		yp = int(b->Ptext.at(a)->yp);
  		np = p.xForm(QPoint(xp, yp - b->LineSp));
  		if (y < np.y())
  			{
  			b->CPos = a;
  			p.end();
				emit ItemTextFont(b->Ptext.at(a)->cfont);
				emit ItemTextSize(b->Ptext.at(a)->csize);
  			return;
  			}
  		}
  	b->CPos = b->Ptext.count();
  	p.end();
  	}
}

void Page::slotDoCurs(bool draw)
{
  if (SelItem.count() != 0)
  	{
  	PageItem *b = Items.at(*SelItem.at(0));
  	if ((b->PType != 4) || (b->Ptext.count() == 0)) { return; }
  	QPainter p;
  	QFont ffo;
  	int xp, yp, yp1;
  	p.begin(this);
		Transform(b, &p);
		TransformM(b, &p);
		if (b->CPos > 0)
			{
			ffo = doku->UsedFonts[b->Ptext.at(b->CPos-1)->cfont];
			ffo.setPointSize(b->Ptext.at(b->CPos-1)->csize);
			p.setFont(ffo);
  		xp = int(b->Ptext.at(b->CPos-1)->xp);
  		if (b->Ptext.at(b->CPos-1)->ch != 13)
  			{ xp += p.fontMetrics().width(QChar(b->Ptext.at(b->CPos-1)->ch)); }
  		yp = int(b->Ptext.at(b->CPos-1)->yp);
  		yp1 = yp - b->Ptext.at(b->CPos-1)->csize;
  		}
  	else
			{
  		xp = int(b->Ptext.at(b->CPos)->xp);
  		yp = int(b->Ptext.at(b->CPos)->yp);
  		yp1 = yp - b->Ptext.at(b->CPos)->csize;
  		}
  	if (draw)
  		{
  		p.setPen(black);
  		CursVis = false;
  		}
  	else
  		{
  		p.setPen(white);
  		CursVis = true;
  		}
  	if (yp1 < int(b->Height))
  		{
  		p.drawLine(xp, yp1, xp, yp);
  		}
  	p.end();
  	}
}

void Page::BlinkCurs()
{
	slotDoCurs(CursVis);
}

void Page::MarkClip(PageItem *b)
{
	float sc = doku->Scale;
	uint a;
	int x, y, d;
	QPainter p;
	p.begin(this);
	Transform(b, &p);
	TransformM(b, &p);
	p.setPen(blue);
	p.setBrush(NoBrush);
	p.drawPolygon(b->Clip);
	for (a=0; a<b->Clip.size(); a++)
		{
		b->Clip.point(a, &x, &y);
		d = a;
		if (d == ClRe)
			{
			p.setPen(red);
			p.setBrush(red);
			}
		else
			{
			p.setPen(blue);
			p.setBrush(blue);
			}
		p.drawRect(x-3/sc, y-3/sc, 6/sc, 6/sc);
		}
	p.end();
}

void Page::SetFrameRect()
{
	ClRe = -1;
  if (SelItem.count() != 0)
  	{
  	PageItem *b = Items.at(*SelItem.at(0));
  	b->Clip.setPoints(4, 0,0, int(b->Width),0, int(b->Width),int(b->Height), 0,int(b->Height));
  	repaint();
  	}
}

void Page::SetFrameOval()
{
	ClRe = -1;
  if (SelItem.count() != 0)
  	{
  	PageItem *b = Items.at(*SelItem.at(0));
  	b->Clip.makeEllipse(0, 0, b->Width, b->Height);
  	repaint();
  	}
}

void Page::ChLineWidth(int w)
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->Pwidth = w;
			}
		repaint();
		}
}

void Page::ChLineArt(PenStyle w)
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->PLineArt = w;
			}
		repaint();
		}
}

void Page::ChLineSpa(int w)
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->LineSp = w;
			}
		repaint();
		}
}


void Page::ChLocalXY(float x, float y)
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->LocalX = x;
			Items.at(*SelItem.at(a))->LocalY = y;
			}
		repaint();
		}
}

void Page::ChLocalSc(float x, float y)
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->LocalScX = x;
			Items.at(*SelItem.at(a))->LocalScY = y;
			}
		repaint();
		}
}

void Page::ItemFont(QString fon)
{
	uint a;
	PageItem *b = Items.at(*SelItem.at(0));
	b->IFont = fon;
	if ((b->HasSel) && (doku->AppMode == 7))
		{
		if (b->Ptext.count() != 0)
			{
			for (a = 0; a < b->Ptext.count(); a++)
				{
				if (b->Ptext.at(a)->cselect)
					{
					b->Ptext.at(a)->cfont = fon;
					}
				}
			b->Dirty = true;
			b->paintObj();
			}
		}
}

void Page::ItemPen(QString farbe)
{
	uint a, b;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
  		if ((Items.at(*SelItem.at(a))->PType == 4) && (doku->AppMode == 7))
  			{
				for (b=0; b<Items.at(*SelItem.at(a))->Ptext.count(); b++)
					{
					if (Items.at(*SelItem.at(a))->Ptext.at(b)->cselect)
						{
						Items.at(*SelItem.at(a))->Ptext.at(b)->ccolor = farbe;
						}
					}
				}
			Items.at(*SelItem.at(a))->Pcolor2 = farbe;
			}
		repaint();
		}
}

void Page::ItemBrush(QString farbe)
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->Pcolor = farbe;
			emit ItemFarben(Items.at(*SelItem.at(a))->Pcolor2,
											Items.at(*SelItem.at(a))->Pcolor,
											Items.at(*SelItem.at(a))->Shade2,
											Items.at(*SelItem.at(a))->Shade);
			}
		repaint();
		}
}

void Page::ItemBrushShade(int sha)
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->Shade = sha;
			if (sha == 0)
				{
				Items.at(*SelItem.at(a))->Transparent = true;
				}
			else
				{
				Items.at(*SelItem.at(a))->Transparent = false;
				}
			emit ItemFarben(Items.at(*SelItem.at(a))->Pcolor2,
											Items.at(*SelItem.at(a))->Pcolor,
											Items.at(*SelItem.at(a))->Shade2,
											Items.at(*SelItem.at(a))->Shade);
			}
		repaint();
		}
}

void Page::ItemPenShade(int sha)
{
	uint a, b;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
  		if ((Items.at(*SelItem.at(a))->PType == 4) && (doku->AppMode == 7))
  			{
				for (b=0; b<Items.at(*SelItem.at(a))->Ptext.count(); b++)
					{
					if (Items.at(*SelItem.at(a))->Ptext.at(b)->cselect)
						{
						Items.at(*SelItem.at(a))->Ptext.at(b)->cshade = sha;
						}
					}
				}
			Items.at(*SelItem.at(a))->Shade2 = sha;
			}
		repaint();
		}
}

void Page::chFSize(int size)
{
	uint a;
	PageItem *b = Items.at(*SelItem.at(0));
	b->ISize = size;
	if ((b->HasSel) && (doku->AppMode == 7))
		{
		if (b->Ptext.count() != 0)
			{
			for (a = 0; a < b->Ptext.count(); a++)
				{
				if (b->Ptext.at(a)->cselect)
					{
					b->Ptext.at(a)->csize = size;
					}
				}
			b->Dirty = true;
			b->paintObj();
			}
		}
	emit ItemTextSize(Items.at(*SelItem.at(0))->ISize);
}

void Page::QueryFarben()
{
	PageItem *b;
  if (SelItem.count() != 0)
  	{
  	b = Items.at(*SelItem.at(0));
  	emit ItemFarben(b->Pcolor2, b->Pcolor, b->Shade2, b->Shade);
  	}
}

void Page::ToggleBookmark()
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->isBookmark = !Items.at(*SelItem.at(a))->isBookmark;
			}
		}
}

void Page::TogglePic()
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->PicArt = !Items.at(*SelItem.at(a))->PicArt;
			}
		repaint();
		}
}

void Page::FlipImageH()
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->flippedH += 1;
			}
		repaint();
		}
}

void Page::FlipImageV()
{
	uint a;
  if (SelItem.count() != 0)
  	{
  	for (a = 0; a < SelItem.count(); a++)
  		{
			Items.at(*SelItem.at(a))->flippedV += 1;
			}
		repaint();
		}
}

void Page::ToBack()
{
	uint a;
  if ((Items.count() > 1) && (SelItem.count() != 0))
  	{
		PageItem *b = Items.take(*SelItem.at(0));
		Items.prepend(b);
		for (a = 0; a < Items.count(); a++)
			{
			Items.at(a)->ItemNr = a;
			}
		SelItem.clear();
		SelItem.append(&b->ItemNr);
		emit DocChanged();
		}
}

void Page::ToFront()
{
	uint a;
  if ((Items.count() > 1) && (SelItem.count() != 0))
  	{
		PageItem *b = Items.take(*SelItem.at(0));
		Items.append(b);
		for (a = 0; a < Items.count(); a++)
			{
			Items.at(a)->ItemNr = a;
			}
		SelItem.clear();
		SelItem.append(&b->ItemNr);
		emit DocChanged();
		}
}

void Page::LowerItem()
{
	uint a;
  if ((Items.count() > 1) && (SelItem.count() != 0) && (*SelItem.at(0)>0))
  	{
		PageItem *b = Items.take(*SelItem.at(0));
		Items.insert(*SelItem.at(0)-1, b);
		for (a = 0; a < Items.count(); a++)
			{
			Items.at(a)->ItemNr = a;
			}
		SelItem.clear();
		SelItem.append(&b->ItemNr);
		emit DocChanged();
		}
}

void Page::RaiseItem()
{
	uint a;
  if ((Items.count() > 1) && (SelItem.count() != 0) && (*SelItem.at(0)<SelItem.count()-2))
  	{
		PageItem *b = Items.take(*SelItem.at(0));
		Items.insert(*SelItem.at(0)+1, b);
		for (a = 0; a < Items.count(); a++)
			{
			Items.at(a)->ItemNr = a;
			}
		SelItem.clear();
		SelItem.append(&b->ItemNr);
		emit DocChanged();
		}
}

void Page::DeleteItem()
{
	uint a;
  if (SelItem.count() != 0)
  	{
		Items.remove(*SelItem.at(0));
		for (a = 0; a < Items.count(); a++)
			{
			Items.at(a)->ItemNr = a;
			}	
		SelItem.clear();
  	qApp->setOverrideCursor(QCursor(ArrowCursor), 1);
		repaint();
		emit HaveSel(-1);
		emit DocChanged();
		}
}

void Page::Deselect()
{
	uint a;
	for (a = 0; a < doku->ActPage->Items.count(); a++)
		{
		doku->ActPage->Items.at(a)->Select = FALSE;
		}
	SelItem.clear();
	emit HaveSel(-1);
	doku->ActPage->repaint();
}

void Page::PasteItem(struct CLBuf *Buffer, bool loading)
{
	Deselect();
	float x = Buffer->Xpos;
	float y = Buffer->Ypos;
	float w = Buffer->Width;
	float h = Buffer->Height;
	int pw = Buffer->Pwidth;
	int z = 0;
	struct Pti *hg;
	switch (Buffer->PType)
		{
		case 1:
			z = PaintEllipse(x, y, w, h, pw, Buffer->Pcolor, Buffer->Pcolor2);
			break;
		case 2:
			z = PaintPict(x, y, w, h);
			Items.at(z)->LocalScX = Buffer->LocalScX;
			Items.at(z)->LocalScY = Buffer->LocalScY;
			Items.at(z)->LocalX = Buffer->LocalX;
			Items.at(z)->LocalY = Buffer->LocalY;
			Items.at(z)->Pfile = Buffer->Pfile;
   		LoadPict(Items.at(z)->Pfile, z);
			Items.at(z)->PicArt = Buffer->PicArt;
			Items.at(z)->flippedH = Buffer->flippedH;
			Items.at(z)->flippedV = Buffer->flippedV;
			Items.at(z)->BBoxX = Buffer->BBoxX;
			Items.at(z)->BBoxH = Buffer->BBoxH;
			break;
		case 3:
			z = PaintRect(x, y, w, h, pw, Buffer->Pcolor, Buffer->Pcolor2);
			break;
		case 4:
			z = PaintText(x, y, w, h, 1, Buffer->Pcolor2);
			if (Buffer->Ptext != "")
				{
				QTextStream t(&Buffer->Ptext, IO_ReadOnly);
				QString cc;
				while (!t.atEnd())
					{
					cc = t.readLine();
					QStringList wt;
					QStringList::Iterator it;
					wt = QStringList::split("\t", cc);
					it = wt.begin();
					hg = new Pti;
					hg->ch = (*it).toInt();
					it++;
					hg->cfont = *it;
					it++;
					hg->csize = (*it).toInt();
					it++;
					hg->ccolor = *it;
					it++;
					hg->cextra = (*it).toInt();
					it++;
					hg->cshade = (*it).toInt();
					hg->cselect = false;
					Items.at(z)->Ptext.append(hg);
					}
				}
				Items.at(z)->LineSp = Buffer->LineSp;
			break;
		case 5:
			z = PaintLine(x, y, w, h, pw, Buffer->Pcolor2);
			break;
		}
	PageItem* b = Items.at(z);
	b->Shade = Buffer->Shade;
	b->Shade2 = Buffer->Shade2;
	b->Rot = Buffer->Rot;
	b->Extra = Buffer->Extra;
	b->PLineArt = PenStyle(Buffer->PLineArt);
	b->isPrintable = Buffer->isPrintable;
	b->isBookmark = Buffer->isBookmark;
	b->Transparent = Buffer->Transparent;
	b->Clip = Buffer->Clip.copy();
	if (!loading)
		{	
		b->Select = true;
		b->paintObj();
		SelItem.append(&b->ItemNr);
		emit HaveSel(b->PType);
		emit ItemPos(b->Xpos, b->Ypos);
		emit ItemGeom(b->Width, b->Height);
		emit SetAngle(b->Rot);
		emit SetSizeValue(b->Pwidth);
		emit SetLineArt(b->PLineArt);
		emit SetLocalValues(b->LocalScX, b->LocalScY, b->LocalX, b->LocalY );
		emit ItemFarben(b->Pcolor2, b->Pcolor, b->Shade2, b->Shade);
		emit ItemTextAttr(b->LineSp);
		emit ItemTextFont(b->IFont);
		emit ItemTextSize(b->ISize);
		emit DocChanged();
		repaint();
		}
}

void Page::SetupDraw(int nr)
{
	Items.at(nr)->Select = TRUE;
	mCG = TRUE;
	HowTo = 1;
	qApp->setOverrideCursor(QCursor(SizeFDiagCursor), 1);
	SelItem.clear();
	SelItem.append(&Items.at(nr)->ItemNr);
	Items.at(nr)->paintObj();
	doku->AppMode = 1;
	emit DocChanged();
	PageItem* b = Items.at(nr);
	emit ItemPos(b->Xpos, b->Ypos);
	emit ItemGeom(b->Width, b->Height);
	emit SetAngle(b->Rot);
	emit SetSizeValue(b->Pwidth);
	emit SetLineArt(b->PLineArt);
	emit SetLocalValues(b->LocalScX, b->LocalScY, b->LocalX, b->LocalY );
	emit ItemFarben(b->Pcolor2, b->Pcolor, b->Shade2, b->Shade);
	emit ItemTextAttr(b->LineSp);
	emit ItemTextFont(b->IFont);
	emit ItemTextSize(b->ISize);
}

void Page::AlignObj(bool xa, bool ya, bool Vth, bool Vtv, float xdisp, float ydisp, int xart, int yart)
{
	uint a;
	float xp = 999999;
	float yp = 999999;
	float xm = 0;
	float xd, xo;
	QList<float> ml;
	QPainter p;
	PageItem *b;
	ml.setAutoDelete(true);
	ml.clear();
	if (xa)
		{
		if (Vth)
			{
			for (a=0; a<SelItem.count(); a++)
				{
				p.begin(this);
				b = Items.at(*SelItem.at(a));
				Transform(b, &p);
				switch (xart)
					{
					case 0:
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().x()));
						break;
					case 1:
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().x()+QRegion(p.xForm(b->Clip)).boundingRect().width() / 2));
						break;
					case 2:
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().right()));
						break;
					}
				p.end();
				}
			xp = 99999;
			xm = 0;
			for (a=0; a<SelItem.count(); a++)
				{
				xp = QMIN(*ml.at(a), xp);
				xm = QMAX(*ml.at(a), xm);
				}
			xd = (xm - xp) / (SelItem.count()-1);
			for	(a=0; a<SelItem.count(); a++)
				{
				xo = *ml.at(a);
				MoveItem((xp + xd * a) - xo, 0, *SelItem.at(a));
				}
			ml.clear();
			}
		else
			{
			switch (xart)
				{
				case 0:
					for (a=0; a<SelItem.count(); a++)
						{
						b = Items.at(*SelItem.at(a));
						p.begin(this);
						Transform(b, &p);
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().x()));
						p.end();
						}
					xp = 99999;
					for (a=0; a<SelItem.count(); a++)
						{
						xp = QMIN(*ml.at(a), xp);
						}
					for	(a=0; a<SelItem.count(); a++)
						{
						MoveItem(xp - *ml.at(a) + a*xdisp, 0, *SelItem.at(a));
						}
					ml.clear();
					break;
				case 1:
					for (a=0; a<SelItem.count(); a++)
						{
						b = Items.at(*SelItem.at(a));
						p.begin(this);
						Transform(b, &p);
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().x()+QRegion(p.xForm(b->Clip)).boundingRect().width() / 2));
						p.end();
						}
					xp = *ml.at(0);
					for	(a=0; a<SelItem.count(); a++)
						{
						MoveItem(xp - *ml.at(a) + a*xdisp, 0, *SelItem.at(a));
						}
					ml.clear();
					break;
				case 2:
					xp = 0;
					for (a=0; a<SelItem.count(); a++)
						{
						b = Items.at(*SelItem.at(a));
						p.begin(this);
						Transform(b, &p);
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().right()));
						p.end();
						}
					for (a=0; a<SelItem.count(); a++)
						{
						xp = QMAX(*ml.at(a), xp);
						}
					for	(a=0; a<SelItem.count(); a++)
						{
						MoveItem(xp - *ml.at(a) + a*xdisp, 0, *SelItem.at(a));
						}
					ml.clear();
					break;
				}
			}
		}
	if (ya)
		{
		if (Vtv)
			{
			for (a=0; a<SelItem.count(); a++)
				{
				p.begin(this);
				b = Items.at(*SelItem.at(a));
				Transform(b, &p);
				switch (yart)
					{
					case 0:
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().y()));
						break;
					case 1:
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().y()+QRegion(p.xForm(b->Clip)).boundingRect().height() / 2));
						break;
					case 2:
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().bottom()));
						break;
					}
				p.end();
				}
			xp = 99999;
			xm = 0;
			for (a=0; a<SelItem.count(); a++)
				{
				xp = QMIN(*ml.at(a), xp);
				xm = QMAX(*ml.at(a), xm);
				}
			xd = (xm - xp) / (SelItem.count()-1);
			for	(a=0; a<SelItem.count(); a++)
				{
				xo = *ml.at(a);
				MoveItem(0, (xp + xd * a) - xo, *SelItem.at(a));
				}
			ml.clear();
			}
		else
			{
			switch (yart)
				{
				case 0:
					for (a=0; a<SelItem.count(); a++)
						{
						b = Items.at(*SelItem.at(a));
						p.begin(this);
						Transform(b, &p);
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().y()));
						p.end();
						}
					yp = 99999;
					for (a=0; a<SelItem.count(); a++)
						{
						yp = QMIN(*ml.at(a), yp);
						}
					for	(a=0; a<SelItem.count(); a++)
						{
						MoveItem(0, yp - *ml.at(a) + a*ydisp, *SelItem.at(a));
						}
					ml.clear();
					break;
				case 1:
					for (a=0; a<SelItem.count(); a++)
						{
						b = Items.at(*SelItem.at(a));
						p.begin(this);
						Transform(b, &p);
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().y()+QRegion(p.xForm(b->Clip)).boundingRect().height() / 2));
						p.end();
						}
					xp = *ml.at(0);
					for	(a=0; a<SelItem.count(); a++)
						{
						MoveItem(0, xp - *ml.at(a) + a*ydisp, *SelItem.at(a));
						}
					ml.clear();
					break;
				case 2:
					yp = 0;
					for (a=0; a<SelItem.count(); a++)
						{
						b = Items.at(*SelItem.at(a));
						p.begin(this);
						Transform(b, &p);
						ml.append(new float(QRegion(p.xForm(b->Clip)).boundingRect().bottom()));
						p.end();
						}
					for (a=0; a<SelItem.count(); a++)
						{
						yp = QMAX(*ml.at(a), yp);
						}
					for	(a=0; a<SelItem.count(); a++)
						{
						MoveItem(0, yp - *ml.at(a) + a*ydisp, *SelItem.at(a));
						}
					ml.clear();
					break;
				}
			}
		}
}

/** Zeichnet eine Ellipse */
int Page::PaintEllipse(float x, float y, float b, float h, int w, QString fill, QString outline)
{
	PageItem* ite = new PageItem(this, 1, x, y, b, h, w, fill, outline, doku);
	Items.append(ite);
	ite->ItemNr = Items.count()-1;
	if (!doku->loading)
		{
		ite->paintObj();
		}
	return ite->ItemNr;
}

/** Zeichnet einen Bildrahmen */
int Page::PaintPict(float x, float y, float b, float h)
{
	PageItem* ite = new PageItem(this, 2, x, y, b, h, 1, "Black", "Black", doku);
	Items.append(ite);
	ite->ItemNr = Items.count()-1;
	if (!doku->loading)
		{
		ite->paintObj();
		}
	return ite->ItemNr;
}

/** Zeichnet ein Rechteck */
int Page::PaintRect(float x, float y, float b, float h, int w, QString fill, QString outline)
{
	PageItem* ite = new PageItem(this, 3, x, y, b, h, w, fill, outline, doku);
	Items.append(ite);
	ite->ItemNr = Items.count()-1;
	if (!doku->loading)
		{
		ite->paintObj();
		}
	return ite->ItemNr;
}

/** Zeichnet einen Textrahmen */
int Page::PaintText(float x, float y, float b, float h, int w, QString outline)
{
	if (w == 0) { w = 1; }
	PageItem* ite = new PageItem(this, 4, x, y, b, h, w, "Black", outline, doku);
	Items.append(ite);
	ite->ItemNr = Items.count()-1;
	ite->Shade = 0;
	if (!doku->loading)
		{
		ite->paintObj();
		}
	return ite->ItemNr;
}

/** Zeichnet eine Linie */
int Page::PaintLine(float x, float y, float b, float h, int w, QString outline)
{
	if (w == 0) { w = 1; }
	PageItem* ite = new PageItem(this, 5, x, y, b, h, w, "Black", outline, doku);
	Items.append(ite);
	ite->ItemNr = Items.count()-1;
	if (!doku->loading)
		{
		ite->paintObj();
		}
	return ite->ItemNr;
}

void Page::LoadPict(QString fn, int ItNr)
{
	QString tmp, dummy, cmd1, cmd2;
	float x, y, b, h;
	bool found = false;
	bool Valid;
	QFileInfo fi = QFileInfo(fn);
	QString ext = fi.extension(false).lower();
	if ((ext == "eps") || (ext == "ps"))
		{
		QFile f(fn);
		if (f.open(IO_ReadOnly))
			{
			QTextStream ts(&f);
			while (!ts.atEnd())
				{
				tmp = ts.readLine();
				if (tmp.startsWith("%%BoundingBox"))
					{
					found = true;
					break;
					}
				}	
			f.close();
			if (found)
				{
				Valid = false;
				QTextStream ts2(&tmp, IO_ReadOnly);
				ts2 >> dummy >> x >> y >> b >> h;
				cmd1 = "gs -q -dNOPAUSE -sDEVICE=pnm -r72 -sOutputFile=- -g";
				cmd2 = " -c showpage -c quit";
				FILE *fp = popen(cmd1 + tmp.setNum(b) + "x" + tmp.setNum(h) + " " + fn + cmd2, "r");
				if (fp == 0)
					{
					Items.at(ItNr)->Pfile = fn;
					Items.at(ItNr)->PicAvail = false;
					Items.at(ItNr)->PicArt = false;
					return;
					}
				QTextStream ts3(fp, IO_ReadOnly);
				tmp = "";
				tmp = ts3.read();
				if (tmp != "")
					Valid = true;
				QCString tmp2 = QCString(tmp);
				if (Valid)
					{
					QByteArray by = QByteArray(tmp2);
					QPixmap im = QPixmap(by);
					im.setMask(im.createHeuristicMask());
					QImage im2 = im.convertToImage();
					Items.at(ItNr)->pixm = im2.copy(x, 0, b-x, h-y);
					Items.at(ItNr)->Pfile = fn;
					Items.at(ItNr)->PicAvail = true;
					Items.at(ItNr)->PicArt = true;
					Items.at(ItNr)->BBoxX = x;
					Items.at(ItNr)->BBoxH = h;
					}
				else
					{
					Items.at(ItNr)->Pfile = fn;
					Items.at(ItNr)->PicAvail = false;
					Items.at(ItNr)->PicArt = false;
					}
				pclose(fp);
				}
			}
		else
			{
			Items.at(ItNr)->Pfile = fn;
			Items.at(ItNr)->PicAvail = false;
			Items.at(ItNr)->PicArt = false;
			}
		}
	else
		{
		if (Items.at(ItNr)->pixm.load(fn))
			{
			Items.at(ItNr)->Pfile = fn;
			Items.at(ItNr)->PicAvail = true;
			Items.at(ItNr)->PicArt = true;
			Items.at(ItNr)->BBoxX = 0;
			Items.at(ItNr)->BBoxH = Items.at(ItNr)->pixm.height();
			}
		else
			{
			Items.at(ItNr)->Pfile = fn;
			Items.at(ItNr)->PicAvail = false;
			Items.at(ItNr)->PicArt = false;
			}
		}
	Items.at(ItNr)->flippedH = 0;
	Items.at(ItNr)->flippedV = 0;
	emit DocChanged();
}


