/***************************************************************************
                          pageitem.cpp  -  description
                             -------------------
    begin                : Sat Apr 7 2001
    copyright            : (C) 2001 by Franz Schmid
    email                : Franz.Schmid@altmuehlnet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pageitem.h"
#include <qpainter.h>
#include <qpen.h>
#include <qfontmetrics.h>
#include <qregion.h>
#include <qpoint.h>
#include <qfileinfo.h>
#include "page.h"

PageItem::PageItem(QWidget *pa, int art, float x, float y, float w, float h, int w2, QString fill, QString outline, ScribusDoc *doc) : QObject(pa)
{
	Xpos = x;
	Ypos = y;
	Width = w;
	Height = h;
	PType = art;
	Rot = 0;
	Parent = pa;
	Doc = doc;
	Pcolor = fill;
	Pcolor2 = outline;
	Shade = 100;
	Shade2 = 100;
	Pwidth = w2;
	PLineArt = Doc->DLineArt;
	if ((art==2) || (art == 4)) { Transparent = true; }
	else { Transparent = false; }
	Select = false;
	QFont ffo;
	QPainter p;
	p.begin(Parent);
	ffo = Doc->UsedFonts[Doc->Dfont];
	ffo.setPointSize(Doc->Dsize);
	p.setFont(ffo);
	IFont = Doc->Dfont;
	ISize = Doc->Dsize;
	LineSp = p.fontMetrics().lineSpacing();
	p.end();
	CurX = 0;
	CurY = 0;
	CPos = 0;
	Extra = 1;
	Ptext.clear();
	Ptext.setAutoDelete(true);
	Pfile = "";
	pixm = QImage();
	LocalScX = 1;
	LocalScY = 1;
	LocalX = 0;
	LocalY = 0;
	flippedH = 0;
	flippedV = 0;
	BBoxX = 0;
	BBoxH = 0;
	if ((art == 4) || (art == 2))
		{
		Frame = true;
		}
	else
		{
		Frame = false;
		}
	if (art == 1)
		{
		Clip.makeEllipse(0, 0, int(w), int(h));
		}
	else
		{
		Clip.setPoints(4, 0,0, int(w),0, int(w),int(h), 0,int(h));
		}
	OwnPage = Doc->ActPage;
	isPrintable = true;
	isBookmark = false;
	HasSel = false;
}
PageItem::~PageItem(){
}
/** Zeichnet das Item */
void PageItem::paintObj(QRect e)
{
	QPainter p, pp;
	QPoint pt1, pt2;
	QRegion cm;
	QFont ffo;
	uint a;
	int wide, desc, asce, xp, yp, leng, h, s, sneu, v;
	QColor tmp;
	struct Pti *hl;
	p.begin(Parent);
	if (!e.isEmpty())
		{
		p.setClipRect(e);
		}
	p.translate(int(Xpos*Doc->Scale), int(Ypos*Doc->Scale));
	p.scale(double(Doc->Scale), double(Doc->Scale));
	p.rotate(double(Rot));
	if (flippedH % 2 != 0)
		{
		p.translate(Width, 0);
		p.scale(-1, 1);
		}
	if (flippedV % 2 != 0)
		{
		p.translate(0, Height);
		p.scale(1, -1);
		}
	Doc->PageColors[Pcolor].hsv(&h, &s, &v);
	sneu = s * Shade / 100;
	tmp.setHsv(h, sneu, v);
	p.setBrush(tmp);
	Doc->PageColors[Pcolor2].hsv(&h, &s, &v);
	sneu = s * Shade2 / 100;
	tmp.setHsv(h, sneu, v);
	p.setPen(QPen(tmp, int(Pwidth*Doc->Scale), PLineArt, FlatCap, MiterJoin));
	switch (PType)
		{
		case 1:
			if (Pwidth == 0) { p.setPen(NoPen); }
			if (Transparent) { p.setBrush(NoBrush); }
			p.drawEllipse(0, 0, int(Width), int(Height));
			break;
		case 2:
			p.setClipRegion(QRegion(p.xForm(Clip)));
			if (!Transparent)
				{
				p.drawPolygon(Clip);
				}
			if (Pfile == "")
				{
				p.setPen(black);
				p.drawLine(0, 0, Width, Height);
				p.drawLine(0, Height, Width, 0);
				}
			else
				{
				if (!PicArt)
					{
					p.setPen(black);
					p.drawLine(0, 0, Width, Height);
					p.drawLine(0, Height, Width, 0);
					if (PicAvail)
						{
						p.setPen(blue);
						}
					else
						{
						p.setPen(red);
						}
					p.save();
					if (flippedH % 2 != 0)
						{
						p.translate(Width, 0);
						p.scale(-1, 1);
						}
					if (flippedV % 2 != 0)
						{
						p.translate(0, Height);
						p.scale(1, -1);
						}
					p.setBackgroundMode(OpaqueMode);
					p.setFont(QFont("Helvetica", 18));
					QFileInfo fi = QFileInfo(Pfile);
					leng = p.fontMetrics().width(fi.fileName());
					xp = int(Width / 2 - leng / 2);
					yp = int(Height / 2 + p.fontMetrics().height() / 2);
					p.drawText(xp, yp, fi.fileName());
					p.restore();
					}
				else
					{
					p.save();
					p.scale(LocalScX, LocalScY);
					if (!Transparent)
						{
						p.eraseRect(0, 0, Width, Height);
						}
					p.setBackgroundMode(TransparentMode);
					p.drawImage(LocalX, LocalY, pixm);
					p.restore();
					}
				}
			break;
		case 3:
			if (Pwidth == 0) { p.setPen(NoPen); }
			if (Transparent) { p.setBrush(NoBrush); }
			p.drawRect(0, 0, int(Width), int(Height));
			break;
		case 4:
			if ((Ptext.count() != 0) || (Dirty))
				{
				QRegion cl = QRegion(p.xForm(Clip));
				if ((OwnPage->Items.count()-1) > ItemNr)
					{
					for (a=ItemNr+1; a<OwnPage->Items.count(); a++)
						{
						pp.begin(Parent);
						pp.translate(OwnPage->Items.at(a)->Xpos*Doc->Scale, OwnPage->Items.at(a)->Ypos*Doc->Scale);
						pp.scale(Doc->Scale, Doc->Scale);
						pp.rotate(OwnPage->Items.at(a)->Rot);
						cm = QRegion(pp.xForm(OwnPage->Items.at(a)->Clip));
						pp.end();
						cl = cl.subtract(cm);
						}
					}
				p.setClipRegion(cl);
				CurX = 0;
				CurY = LineSp+Extra;
				if ((Doc->AppMode == 7) && (Dirty))
					{
					p.eraseRect(0, 0, Width, Height);
					Dirty = false;
					}
				if (!Transparent)
					{
					p.drawPolygon(Clip);
					}
				for (a = 0; a < Ptext.count(); a++)
					{
					hl = Ptext.at(a);
					if ((hl->ch == 10) || (hl->ch == 13))
						{
						CurY += LineSp;
						CurX = 0;
						hl->xp = CurX;
						hl->yp = CurY;
						continue;
						}
					ffo = Doc->UsedFonts[hl->cfont];
					ffo.setPointSize(hl->csize);
					p.setFont(ffo);
					Doc->PageColors[hl->ccolor].hsv(&h, &s, &v);
					sneu = s * hl->cshade / 100;
					tmp.setHsv(h, sneu, v);
					p.setPen(tmp);
					desc = p.fontMetrics().descent();
					asce = p.fontMetrics().ascent();
					wide = p.fontMetrics().width(QChar(hl->ch));
					pt1 = QPoint(CurX, CurY+desc);
					pt2 = QPoint(CurX, CurY-asce);
					while ((!cl.contains(p.xForm(pt1))) || (!cl.contains(p.xForm(pt2))))
						{
						CurX += 1;
						pt1 = QPoint(CurX, CurY+desc);
						pt2 = QPoint(CurX, CurY-asce);
						if (CurX > Width)
							{
							CurY += LineSp;
							CurX = 0;
							if (CurY > Height)
								{
								break;
								}
							}
						}
					CurX += Extra;
					pt1 = QPoint(CurX+wide+hl->cextra, CurY+desc);
					pt2 = QPoint(CurX+wide+hl->cextra, CurY-asce);
					if ((!cl.contains(p.xForm(pt1))) || (!cl.contains(p.xForm(pt2))))
						{
						CurX = 0;
						CurY += LineSp;
						pt1 = QPoint(CurX+hl->cextra, CurY+desc+Extra);
						pt2 = QPoint(CurX+hl->cextra, CurY-asce);
						while ((!cl.contains(p.xForm(pt1))) || (!cl.contains(p.xForm(pt2))))
							{
							CurX += 1;
							pt1 = QPoint(CurX, CurY+desc);
							pt2 = QPoint(CurX, CurY-asce);
							if (CurX > Width)
								{
								CurY += LineSp;
								CurX = 0;
								if (CurY > Height)
									{
									break;
									}
								}
							CurX += Extra;
							}
						}
					if ((hl->cselect) && (Doc->AppMode == 7))
						{
						p.save();
						p.setPen(NoPen);
            p.setBrush(darkBlue);
            p.drawRect(CurX+hl->cextra, CurY-asce, wide+1, asce+desc);
						p.restore();
						p.setPen(white);
						}
					p.drawText(CurX+hl->cextra, CurY, QChar(hl->ch));
					hl->xp = CurX+hl->cextra;
					hl->yp = CurY;
					CurX += wide+hl->cextra;
					}
				}
				break;
		case 5:
			p.drawLine(0, 0, int(Width), int(Height));
			break;
		}
	p.setClipping(FALSE);
	if (!e.isEmpty())
		{
		p.setClipRect(e);
		}
	if (Frame)
		{
		p.setPen(QPen(black, 1, DotLine, FlatCap, MiterJoin));
		if (isBookmark)
			p.setPen(QPen(blue, 1, DotLine, FlatCap, MiterJoin));
		p.setBrush(NoBrush);
		p.drawPolygon(Clip);
		}
	if ((Select) && (!Doc->EditClip))
		{
		p.setPen(QPen(red, 1, DotLine, FlatCap, MiterJoin));
		p.setBrush(NoBrush);
		p.drawRect(-1, -1, int(Width+1), int(Height+1));
		p.setPen(NoPen);
		p.setBrush(red);
		p.drawRect(-1, -1, 6, 6);
		p.drawRect(int(Width), int(Height), -6, -6);
		p.drawRect(int(Width), -1, -6, 6);
		p.drawRect(-1, int(Height), 6, -6);
		}
	p.end();
}

void PageItem::CopyIt(struct CLBuf *Buffer)
{
	uint a;
	Buffer->PType = PType;
	Buffer->Xpos = Xpos;
	Buffer->Ypos = Ypos;
	Buffer->Width = Width;
	Buffer->Height = Height;
	Buffer->Pwidth = Pwidth;
	Buffer->Pcolor = Pcolor;
	Buffer->Pcolor2 = Pcolor2;
	Buffer->Shade = Shade;
	Buffer->Shade2 = Shade2;
	Buffer->Rot = Rot;
	Buffer->PLineArt = PLineArt;
	Buffer->LineSp = LineSp;
	Buffer->LocalScX = LocalScX;
	Buffer->LocalScY = LocalScY;
	Buffer->LocalX = LocalX;
	Buffer->LocalY = LocalY;
	Buffer->PicArt = PicArt;
	Buffer->flippedH = flippedH;
	Buffer->flippedV = flippedV;
	Buffer->BBoxX = BBoxX;
	Buffer->BBoxH = BBoxH;
	Buffer->isPrintable = isPrintable;
	Buffer->isBookmark = isBookmark;
	Buffer->Transparent = Transparent;
	Buffer->Extra = Extra;
	Buffer->Pfile = Pfile;
	QString Text = "";
	if (Ptext.count() != 0)
		{
		for (a=0; a<Ptext.count(); a++)
			{
			Text += QString::number(Ptext.at(a)->ch)+"\t";
			Text += Ptext.at(a)->cfont+"\t";
			Text += QString::number(Ptext.at(a)->csize)+"\t";
			Text += Ptext.at(a)->ccolor+"\t";
			Text += QString::number(Ptext.at(a)->cextra)+"\t";
			Text += QString::number(Ptext.at(a)->cshade)+'\n';
			}
		}
	Buffer->Ptext = Text;
	Buffer->Clip = Clip.copy();
}
