/***************************************************************************
                          scribus.cpp  -  description
                             -------------------
    begin                : Fre Apr  6 21:09:31 CEST 2001
    copyright            : (C) 2001 by Franz Schmid
    email                : Franz.Schmid@altmuehlnet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qaccel.h>
#include <qapplication.h>
#include <qcolordialog.h>
#include <qcolor.h>
#include <qiconset.h>
#include <qtextstream.h>
#include <qplatinumstyle.h>
#include <qwindowsstyle.h>
#include <qmotifstyle.h>
#include <qmotifplusstyle.h>
#include <qcdestyle.h>
#include "scribus.h"
#include "newfile.h"
#include "page.h"
#include "query.h"
#include "mdup.h"
#include "docinfo.h"
#include "reformdoc.h"
#include "serializer.h"
#include "align.h"
#include "fmitem.h"
#include "fontprefs.h"
#include "missing.h"
#include "prefs.h"
#include "appicon.h"
#include "pslib.h"
#include "stdio.h"
#include "stdlib.h"
#include <qdatastream.h>
#include <pdfopts.h>
//#include <iostream.h>
static const char *Lupe[]={
"16 14 3 1",
". c None",
"# c #000000",
"a c #ffffff",
"................",
"....####........",
"...#....#.......",
"..#..a...#......",
"..#.a....#......",
"..#......#......",
"..#......#......",
"...#....##......",
"....#######.....",
".........###....",
"..........###...",
"...........###..",
"............##..",
"................"};

ScribusApp::ScribusApp()
{
  setCaption(tr("Scribus " VERSION));
  QPixmap image0( ( const char** ) AppIcon );
  setIcon (image0);
  initMenuBar();
  initStatusBar();
  /** Erstelle Fontliste */
  NoFonts = false;
  GetAllFonts();
  BuildFontMenu();
  DefFont = AvailFonts.begin().key();
  DefSize = 12;
  /** Default Farbenliste */
  DColors.clear();
  DColors.insert("White", QColor(white));
  DColors.insert("Black", QColor(black));
	DColors.insert("Blue", QColor(blue));
	DColors.insert("Cyan", QColor(cyan));
	DColors.insert("Green", QColor(green));
	DColors.insert("Red", QColor(red));
	DColors.insert("Yellow", QColor(yellow));
	DColors.insert("Magenta", QColor(magenta));
	DispX = 10;
	DispY = 10;
	Wheelval = 40;
  DminGrid = 20;
  DmajGrid = 100;
  DocNr = 1;
  DminColor = QColor(green);
  DmajColor = QColor(green);
  GUI = 1;
  HaveDoc = false;
  BufferSet = false;
  BufferSet2 = false;
  Wsh = false;
  Msh = false;
  Csh = false;
  Tsh = false;
  PDFavailable = true;
	resize(610, 600);
	werkz = new Tools(this, "Tool", 0, WStyle_Customize | WStyle_NoBorderEx);
	werkz->Textedit->setEnabled(false);
	werkz->Rotiere->setEnabled(false);
	Mpal = new Mpalette(this, AvailFonts);
	Cpal = new Cpalette(this, "Color", 0, WStyle_Customize | WStyle_DialogBorder);
	Cpal->SetColors(DColors);
	Tpal = new Tree(this, WStyle_Customize | WStyle_DialogBorder);
	ReadPrefs();
	HaveGS = system("gs -h > /dev/null 2>&1");
	HaveConvert = system("convert > /dev/null 2>&1");
	HavePFB = system("pfb2pfa > /dev/null 2>&1");
	connect(werkz, SIGNAL(SetMode(int)), this, SLOT(setAppMode(int)));
	connect(werkz->TA, SIGNAL(Schliessen()), this, SLOT(ToggleTools()));
	connect(Mpal->TA, SIGNAL(Schliessen()), this, SLOT(ToggleMpal()));
	connect(Mpal->X_val, SIGNAL(returnPressed()), this, SLOT(setItemXp()));
	connect(Mpal->Y_val, SIGNAL(returnPressed()), this, SLOT(setItemXp()));
	connect(Mpal->B_val, SIGNAL(returnPressed()), this, SLOT(setItemXp()));
	connect(Mpal->H_val, SIGNAL(returnPressed()), this, SLOT(setItemXp()));
	connect(Mpal->R_val, SIGNAL(returnPressed()), this, SLOT(setItemXp()));
	connect(Mpal->L_val, SIGNAL(returnPressed()), this, SLOT(setItemXp()));
	connect(Mpal, SIGNAL(setLineW(int)), this, SLOT(setItemLineWidth(int)));
	connect(Mpal, SIGNAL(setLSP(int)), this, SLOT(setItemLspS(int)));
	connect(Mpal, SIGNAL(setLineA(PenStyle)), this, SLOT(setLineStipple(PenStyle)));
	connect(Mpal, SIGNAL(setLocalXY(float, float)), this, SLOT(setItemLocalXY(float, float)));
	connect(Mpal, SIGNAL(setLocalSc(float, float)), this, SLOT(setItemLocalSc(float, float)));
	connect(Mpal, SIGNAL(FlipH()), this, SLOT(setImageH()));
	connect(Mpal, SIGNAL(FlipV()), this, SLOT(setImageV()));
	connect(Mpal, SIGNAL(NewFont(QString)), this, SLOT(SetNewFont(QString)));
	connect(Mpal, SIGNAL(NewFSize(int)), this, SLOT(setFontSize(int)));
	connect(Cpal, SIGNAL(Schliessen()), this, SLOT(ToggleCpal()));
	connect(Cpal, SIGNAL(NewPen(QString)), this, SLOT(setPenFarbe(QString)));
	connect(Cpal, SIGNAL(NewBrush(QString)), this, SLOT(setBrushFarbe(QString)));
	connect(Cpal, SIGNAL(NewPenShade(int)), this, SLOT(setPenShade(int)));
	connect(Cpal, SIGNAL(NewBrushShade(int)), this, SLOT(setBrushShade(int)));
	connect(Cpal, SIGNAL(QueryItem()), this, SLOT(GetBrushPen()));
	connect(Tpal, SIGNAL(Schliessen()), this, SLOT(ToggleTpal()));
	connect(Tpal, SIGNAL(SelectElement(int, int)), this, SLOT(SelectFromOutl(int, int)));
	connect(Tpal, SIGNAL(SelectSeite(int)), this, SLOT(SelectFromOutlS(int)));
  connect(ColorMenu, SIGNAL(activated(int)), this, SLOT(setItemFarbe(int)));
  connect(ShadeMenu, SIGNAL(activated(int)), this, SLOT(setItemShade(int)));
  connect(FontMenu, SIGNAL(activated(int)), this, SLOT(setItemFont(int)));
  connect(SizeTMenu, SIGNAL(activated(int)), this, SLOT(setItemFSize(int)));
	connect(this, SIGNAL(TextIFont(QString)), this, SLOT(AdjustFontMenu(QString)));
	connect(this, SIGNAL(TextIFont(QString)), Mpal, SLOT(AdjustFMenu(QString)));
	connect(this, SIGNAL(TextISize(int)), this, SLOT(setFSizeMenu(int)));
	connect(this, SIGNAL(TextISize(int)), Mpal, SLOT(setFSizeMenu(int)));
}

ScribusApp::~ScribusApp()
{
}

void ScribusApp::initMenuBar()
{
	int a;
	fileMenu=new QPopupMenu();
	fileMenu->insertItem(tr("New"), this, SLOT(slotFileNew()));
	fileMenu->insertItem(tr("Open..."), this, SLOT(slotDocOpen()));
	fileMenu->insertSeparator();
	fid1 = fileMenu->insertItem(tr("Close"), this, SLOT(slotFileClose()));
	fileMenu->setItemEnabled(fid1, 0);
	fid4 = fileMenu->insertItem(tr("Save"), this, SLOT(slotFileSave()));
	fileMenu->setItemEnabled(fid4, 0);
	fid5 = fileMenu->insertItem(tr("Save as..."), this, SLOT(slotFileSaveAs()));
	fileMenu->setItemEnabled(fid5, 0);
	fileMenu->insertSeparator();
	fid2 = fileMenu->insertItem(tr("Get Text/Picture..."), this, SLOT(slotFileOpen()));
	fileMenu->setItemEnabled(fid2, 0);
	fid3 = fileMenu->insertItem(tr("Save Text..."), this, SLOT(SaveText()));
	fileMenu->setItemEnabled(fid3, 0);
	fid8 = fileMenu->insertItem(tr("Save Page as EPS..."), this, SLOT(SaveAsEps()));
	fileMenu->setItemEnabled(fid8, 0);
	fid10 = fileMenu->insertItem(tr("Save as PDF..."), this, SLOT(SaveAsPDF()));
	fileMenu->setItemEnabled(fid10, 0);
	fileMenu->insertSeparator();
	fid6 = fileMenu->insertItem(tr("Document Info..."), this, SLOT(InfoDoc()));
	fileMenu->setItemEnabled(fid6, 0);
	fid7 = fileMenu->insertItem(tr("Document Setup..."), this, SLOT(SetupDoc()));
	fileMenu->setItemEnabled(fid7, 0);
	fid9 = fileMenu->insertItem(tr("Print..."), this, SLOT(slotFilePrint()));
	fileMenu->setItemEnabled(fid9, 0);
	fileMenu->insertSeparator();
	fileMenu->insertItem(tr("Quit"), this, SLOT(slotFileQuit()));
	editMenu = new QPopupMenu();
	edid1 = editMenu->insertItem(tr("Cut"), this , SLOT(slotEditCut()));
	edid2 = editMenu->insertItem(tr("Copy"), this , SLOT(slotEditCopy()));
	edid3 = editMenu->insertItem(tr("Paste"), this , SLOT(slotEditPaste()));
	edid4 = editMenu->insertItem(tr("Clear"), this, SLOT(DeleteText()));
	edid5 = editMenu->insertItem(tr("Select all"), this, SLOT(SelectAll()));
	editMenu->insertSeparator();
	editMenu->insertItem(tr("Colors..."), this , SLOT(slotEditColors()));
	editMenu->insertItem(tr("Fonts..."), this , SLOT(slotFontOrg()));
	editMenu->insertItem(tr("Preferences..."), this , SLOT(slotPrefsOrg()));
	editMenu->setItemEnabled(edid1, 0);
	editMenu->setItemEnabled(edid2, 0);
	editMenu->setItemEnabled(edid3, 0);
	editMenu->setItemEnabled(edid4, 0);
	editMenu->setItemEnabled(edid5, 0);
	StilMenu = new QPopupMenu();
	ObjMenu = new QPopupMenu();
	ObjMenu->insertItem(tr("Duplicate"), this, SLOT(ObjektDup()));
	ObjMenu->insertItem(tr("Multiple Duplicate"), this, SLOT(ObjektDupM()));
	ObjMenu->insertItem(tr("Delete"), this, SLOT(DeleteObjekt()));
	ObjMenu->insertSeparator();
	ObjMenu->insertItem(tr("Send to Back"), this, SLOT(Objekt2Back()));
	ObjMenu->insertItem(tr("Bring to Front"), this, SLOT(Objekt2Front()));
	ObjMenu->insertItem(tr("Lower"), this, SLOT(ObjektLower()));
	ObjMenu->insertItem(tr("Raise"), this, SLOT(ObjektRaise()));
	DistM = ObjMenu->insertItem(tr("Distribute/Align..."), this, SLOT(ObjektAlign()));
	ObjMenu->insertSeparator();
	ShapeMenu = new QPopupMenu();
	ShapeMenu->insertItem(tr("Rectangle"), this, SLOT(RectFrame()));
	ShapeMenu->insertItem(tr("Oval"), this, SLOT(OvalFrame()));
	ShapeEdit = ShapeMenu->insertItem(tr("Edit Frame"), this, SLOT(ToggleFrameEdit()));
	ShapeM = ObjMenu->insertItem(tr("Shape"), ShapeMenu);
	ObjMenu->setItemEnabled(ShapeM, 0);
	ObjMenu->setItemEnabled(DistM, 0);
	pageMenu = new QPopupMenu();
	pageMenu->insertItem(tr("Insert..."), this, SLOT(slotNewPage()));
	pgmd = pageMenu->insertItem(tr("Delete..."), this, SLOT(DeletePage()));
	pageMenu->setItemEnabled(pgmd, 0);
	viewMenu=new QPopupMenu();
	viewMenu->insertItem(tr("Fit in Window"), this, SLOT(slotZoomFit()));
	viewMenu->insertItem("50%", this, SLOT(slotZoom50()));
	viewMenu->insertItem("75%", this, SLOT(slotZoom75()));
	viewMenu->insertItem(tr("Actual Size"), this, SLOT(slotZoom100()));
	viewMenu->insertItem("200%", this, SLOT(slotZoom200()));
	viewMenu->insertItem(tr("Thumbnails"), this, SLOT(slotZoom20()));
	viewMenu->insertSeparator();
	Markers = viewMenu->insertItem(tr("Hide Margins"), this, SLOT(ToggleMarks()));
	Bilder = viewMenu->insertItem(tr("Hide Images"), this, SLOT(TogglePics()));
	Ras = viewMenu->insertItem(tr("Show Guides"), this, SLOT(ToggleRaster()));
	uRas = viewMenu->insertItem(tr("Snap to Guides"), this, SLOT(ToggleURaster()));
	viewMenu->insertSeparator();
	viewTools = viewMenu->insertItem(tr("Show Tools"), this, SLOT(ToggleTools()));
	viewMpal = viewMenu->insertItem(tr("Show Measurements"), this, SLOT(ToggleMpal()));
	viewCpal = viewMenu->insertItem(tr("Show Colors"), this, SLOT(ToggleCpal()));
	viewTpal = viewMenu->insertItem(tr("Show Outline"), this, SLOT(ToggleTpal()));
	for (a=0; a<6; a++)
		{
		viewMenu->setItemEnabled(viewMenu->idAt(a), 0);
		}
	viewMenu->setItemEnabled(Markers, 0);
	viewMenu->setItemEnabled(Bilder, 0);
	viewMenu->setItemEnabled(Ras, 0);
	viewMenu->setItemEnabled(uRas, 0);
/*	testMenu=new QPopupMenu();
	testMenu->insertItem("Test1", this, SLOT(slotTest()));
	testMenu->insertItem("Test2", this, SLOT(slotNewPage()));
	testMenu->insertItem("Ellipsen", this, SLOT(slotTest2()));
	testMenu->insertItem("Rechtecke", this, SLOT(slotTest3()));
	testMenu->insertItem("Linien", this, SLOT(slotTest4()));    */
	helpMenu=new QPopupMenu();
	helpMenu->insertItem(tr("About Scribus"), this, SLOT(slotHelpAbout()));
	helpMenu->insertItem(tr("About Qt"), this, SLOT(slotHelpAboutQt()));
	menuBar()->insertItem(tr("File"), fileMenu);
	menuBar()->insertItem(tr("Edit"), editMenu);
	Stm = menuBar()->insertItem(tr("Style"), StilMenu);
	Obm = menuBar()->insertItem(tr("Item"), ObjMenu);
	pgmm = menuBar()->insertItem(tr("Page"), pageMenu);
	menuBar()->setItemEnabled(Stm, 0);
	menuBar()->setItemEnabled(Obm, 0);
	menuBar()->setItemEnabled(pgmm, 0);
	menuBar()->insertItem(tr("View"), viewMenu);
//	menuBar()->insertItem(tr("Tests"), testMenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem(tr("Help"), helpMenu);
	ColorMenu = new QPopupMenu();
	SizeTMenu = new QPopupMenu();
	SizeTMenu->insertItem(tr("Other..."));
	SizeTMenu->insertItem(" 7 pt");
	SizeTMenu->insertItem(" 9 pt");
	SizeTMenu->insertItem("10 pt");
	SizeTMenu->insertItem("12 pt");
	SizeTMenu->insertItem("14 pt");
	SizeTMenu->insertItem("18 pt");
	SizeTMenu->insertItem("24 pt");
	SizeTMenu->insertItem("36 pt");
	SizeTMenu->insertItem("48 pt");
	SizeTMenu->insertItem("60 pt");
	SizeTMenu->insertItem("72 pt");
	ShadeMenu = new QPopupMenu();
	ShadeMenu->insertItem(tr("Other..."));
	ShadeMenu->insertItem("0 %");
	ShadeMenu->insertItem("10 %");
	ShadeMenu->insertItem("20 %");
	ShadeMenu->insertItem("30 %");
	ShadeMenu->insertItem("40 %");
	ShadeMenu->insertItem("50 %");
	ShadeMenu->insertItem("60 %");
	ShadeMenu->insertItem("70 %");
	ShadeMenu->insertItem("80 %");
	ShadeMenu->insertItem("90 %");
	ShadeMenu->insertItem("100 %");
	FStyleMenu = new QPopupMenu();
	FontMenu = new QPopupMenu();
}

void ScribusApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->message(tr("Ready"), 2000);
}

void ScribusApp::DeleteSel(PageItem *b)
{
	Pti *it;
 	for (it = b->Ptext.first(); it != 0; it = b->Ptext.next())
 		{
 		if (it->cselect)
 			{
 			b->Ptext.remove();
 			it = b->Ptext.prev();
 			if (it == 0)
 				{
 				it = b->Ptext.first();
 				}
 			}
 		}
 	if (b->Ptext.first()->cselect)
 		{
 		b->Ptext.remove();
 		}
 	b->HasSel = false;
 	b->CPos = QMIN(b->CPos, doc->ActPage->oldCp);
 	DisableTxEdit();
}

void ScribusApp::setTBvals(PageItem *b)
{
	if (b->Ptext.count() != 0)
		{
		emit TextIFont(b->Ptext.at(QMIN(b->CPos, int(b->Ptext.count()-1)))->cfont);
 		emit TextISize(b->Ptext.at(QMIN(b->CPos, int(b->Ptext.count()-1)))->csize);
 		b->IFont = b->Ptext.at(QMIN(b->CPos, int(b->Ptext.count()-1)))->cfont;
 		b->ISize = b->Ptext.at(QMIN(b->CPos, int(b->Ptext.count()-1)))->csize;
 		}
}

void ScribusApp::wheelEvent(QWheelEvent *w)
{
	if (w->delta() < 0)
		{
 		view->scrollBy(0, -Wheelval);
		}
	else
		{
 		view->scrollBy(0, Wheelval);
		}
	w->accept();
}

void ScribusApp::keyPressEvent(QKeyEvent *k)
{
	struct Pti *hg;
	int kk = k->key();
	int as = k->ascii();
 	if (HaveDoc)
 		{
 		switch (kk)
 			{
 			case Key_Prior:
 				view->scrollBy(0, -Wheelval);
 				return;
 				break;
 			case Key_Next:
 				view->scrollBy(0, Wheelval);
 				return;
 				break;
 			}
 		if (doc->ActPage->SelItem.count() != 0)
 			{
 			PageItem *b = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0));
 			switch (doc->AppMode)
 				{
 				case 1:
 					switch (kk)
 						{
 						case Key_Delete:
 							doc->ActPage->DeleteItem();
 							break;
 						case Key_Left:
 							doc->ActPage->MoveItem(-1, 0, b->ItemNr);
 							break;
 						case Key_Right:
 							doc->ActPage->MoveItem(1, 0, b->ItemNr);
 							break;
 						case Key_Up:
 							doc->ActPage->MoveItem(0, -1, b->ItemNr);
 							break;
 						case Key_Down:
 							doc->ActPage->MoveItem(0, 1, b->ItemNr);
 						}
 					break;
 				case 7:
 					if (b->PType == 4)
 						{
 						doc->ActPage->slotDoCurs(false);
 						switch (kk)
 							{
 							case Key_Left:
 								b->CPos -= 1;
 								if (b->CPos < 0)
 									{
 									b->CPos = 0;
 									}
 								setTBvals(b);
 								break;
 							case Key_Right:
 								b->CPos += 1;
 								if (b->CPos > int(b->Ptext.count()))
 									{
 									b->CPos -= 1;
 									}
 								setTBvals(b);
 								break;
 							case Key_Delete:
 								if (b->CPos == int(b->Ptext.count())) { return; }
 								if (b->HasSel)
 									{
 									DeleteSel(b);
 									}
 								else
 									{
 									b->Ptext.remove(b->CPos);
 									}
 								setTBvals(b);
 								b->Dirty = true;
 								b->paintObj();
 								break;
 							case Key_Backspace:
 								if (b->HasSel)
 									{
 									DeleteSel(b);
 									}
 								else
 									{
 									if (b->CPos == 0) { return; }
 									b->CPos -= 1;
 									b->Ptext.remove(b->CPos);
 									}
 								setTBvals(b);
 								b->Dirty = true;
 								b->paintObj();
 								break;
 							default:
 								if (b->HasSel)
 									{
 									DeleteSel(b);
 									b->Dirty = true;
 									b->paintObj();
 									}
 								if ((as > 31) || (as == 13))
 									{
 									hg = new Pti;
 									hg->ch = as;
 									hg->cfont = b->IFont;
 									hg->csize = b->ISize;
 									hg->ccolor = b->Pcolor2;
 									hg->cextra = 0;
 									hg->cshade = b->Shade2;
 									hg->cselect = false;
 									b->Ptext.insert(b->CPos, hg);
 									b->CPos += 1;
 									if (b->CPos < int(b->Ptext.count())) { b->Dirty = true; }
 									b->paintObj();
 									}
 								break;
 							}
 						doc->ActPage->slotDoCurs(true);
 						}
 					break;
 				}
 			slotDocCh();
 			}
 		}
}

void ScribusApp::closeEvent(QCloseEvent *ce)
{
	if ((HaveDoc) && doc->isModified())
		{
		if (queryExit())
			{
			SavePrefs();
			ce->accept();
			}
		else
			{
			return;
			}
		}
	else
		{
		SavePrefs();
		ce->accept();
		}
}

bool ScribusApp::queryExit()
{
  bool retw = false;
  int exit=QMessageBox::information(this,
  																	tr("Warning"),
                                    tr("The document has been changed since the last save."),
                                    tr("Save Now"),
                                    tr("Cancel"),
                                    tr("Leave Anyway"),
                                    0, 1);
  switch (exit)
  	{
  	case 0:
  		slotFileSaveAs();
  		retw = true;
  		break;
  	case 1:
  		retw = false;
  		break;
  	case 2:
  		retw = true;
  		break;
  	}
  return retw;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////
float ScribusApp::mm2pts(int mm)
{
	return mm / 25.4 * 72;
}

float ScribusApp::pts2mm(float pts)
{
	return pts / 72 * 25.4;
}

void ScribusApp::slotFileNew()
{
	float b, h, tpr, lr, rr, br, sp, ab;
	bool fp, atf;
	QString cc;
	if (HaveDoc)
		{
		slotFileClose();
		}
	NewDoc* dia = new NewDoc(this, "NEU", TRUE);
	if (dia->exec())
		{
		b = mm2pts(dia->Breite->value());
		h = mm2pts(dia->Hoehe->value());
		tpr = mm2pts(dia->TopR->value());
		lr = mm2pts(dia->LeftR->value());
		rr = mm2pts(dia->RightR->value());
		br = mm2pts(dia->BottomR->value());
		sp = dia->SpinBox10->value();
		ab = mm2pts(dia->Distance->value());
		atf = dia->AutoFrame->isChecked();
		fp = dia->Doppelseiten->isChecked();
		doc = new ScribusDoc();
		doc->AddFont(DefFont, AvailFonts[DefFont].Font);
		doc->Dfont = DefFont;
		doc->Dsize = DefSize;
		doc->minorGrid = DminGrid;
		doc->majorGrid = DmajGrid;
		doc->minorColor = DminColor;
		doc->majorColor = DmajColor;
		doc->DocName = doc->DocName+cc.setNum(DocNr);
		DocNr++;
		HaveDoc = TRUE;
		doc->Scale = 1.0;
		doc->AppMode = 1;
		doc->PageColors = DColors;
		view=new ScribusView(this, doc);
		setCentralWidget(view);
		doc->setPage(b, h, tpr, lr, rr, br, sp, ab, atf, fp);
		slotNewPage();
		HaveNewDoc();
		doc->setUnModified();
  	statusBar()->message(tr("Ready"), 3000);
		}
	delete dia;
}

void ScribusApp::SetupDoc()
{
	int tpr = qRound(pts2mm(doc->PageM.Top));
	int lr = qRound(pts2mm(doc->PageM.Left));
	int rr = qRound(pts2mm(doc->PageM.Right));
	int br = qRound(pts2mm(doc->PageM.Bottom));	
	bool fp = doc->PageFP;
	float tpr2, lr2, rr2, br2;
	ReformDoc* dia = new ReformDoc(this, tpr, lr, rr, br, fp);
	if (dia->exec())
		{
		tpr2 = mm2pts(dia->TopR->value());
		lr2 = mm2pts(dia->LeftR->value());
		rr2 = mm2pts(dia->RightR->value());
		br2 = mm2pts(dia->BottomR->value());
		fp = dia->Doppelseiten->isChecked();
		doc->resetPage(tpr2, lr2, rr2, br2, fp);
		view->reformPages();
		}
	delete dia;
}

void ScribusApp::HaveNewDoc()
{
	int a;
	fileMenu->setItemEnabled(fid1, 1);
	fileMenu->setItemEnabled(fid4, 0);
	fileMenu->setItemEnabled(fid5, 1);
	fileMenu->setItemEnabled(fid6, 1);
	fileMenu->setItemEnabled(fid7, 1);
	fileMenu->setItemEnabled(fid8, 1);
	fileMenu->setItemEnabled(fid9, 1);
	if (PDFavailable)
		fileMenu->setItemEnabled(fid10, 1);
	editMenu->setItemEnabled(edid1, 0);
	editMenu->setItemEnabled(edid2, 0);
	editMenu->setItemEnabled(edid3, 0);
	for (a=0; a<6; a++)
		{
		viewMenu->setItemEnabled(viewMenu->idAt(a), 1);
		}
	viewMenu->setItemEnabled(Markers, 1);
	viewMenu->setItemEnabled(Bilder, 1);
	viewMenu->setItemEnabled(Ras, 1);
	viewMenu->setItemEnabled(uRas, 1);
	viewMenu->setItemChecked(uRas, doc->useRaster);
	menuBar()->setItemEnabled(pgmm, 1);
	CListe::Iterator it;
	QPixmap pm = QPixmap(15, 15);
	a = 0;
	for (it = doc->PageColors.begin(); it != doc->PageColors.end(); ++it)
		{
		pm.fill(doc->PageColors[it.key()]);
		ColorMenu->insertItem(QIconSet(pm), it.key());
		if (it.key() == doc->Dbrush)
			{
			ColorMenu->setItemChecked(ColorMenu->idAt(a), true);
			}
		a++;
		}
	Cpal->SetColors(doc->PageColors);
	setCaption(tr(doc->DocName));
	ShadeMenu->setItemChecked(ShadeMenu->idAt(11), true);
	Mpal->BuildFontMenu(AvailFonts);
	QMap<QString,QFont>::Iterator it3;
	for (it3 = doc->UsedFonts.begin(); it3 != doc->UsedFonts.end(); ++it3)
		{
		a = Mpal->FontMenu->insertItem(new FmItem(it3.key(), it3.data()));
		if (it3.key() == doc->Dfont)
			{
			Mpal->FontMenu->setItemChecked(a, true);
			}
		Mpal->FontID.insert(long(a), &it3.key());
		}
  BuildFontMenu();
}

void ScribusApp::HaveNewSel(int Nr)
{
	switch (Nr)
		{
		case -1:
			fileMenu->changeItem(fid2, tr("Get Text/Picture..."));
			fileMenu->setItemEnabled(fid2, 0);
			fileMenu->setItemEnabled(fid3, 0);
			menuBar()->setItemEnabled(Stm, 0);
			menuBar()->setItemEnabled(Obm, 0);
			ObjMenu->setItemEnabled(ShapeM, 0);
			editMenu->setItemEnabled(edid1, 0);
			editMenu->setItemEnabled(edid2, 0);
			editMenu->setItemEnabled(edid4, 0);
			editMenu->setItemEnabled(edid5, 0);
			StilMenu->clear();
			werkz->Textedit->setEnabled(false);
			werkz->Rotiere->setEnabled(false);
			break;
		case 2:
			fileMenu->changeItem(fid2, tr("Get Picture..."));
			fileMenu->setItemEnabled(fid2, 1);
			editMenu->setItemEnabled(edid1, 1);
			editMenu->setItemEnabled(edid2, 1);
			editMenu->setItemEnabled(edid4, 0);
			editMenu->setItemEnabled(edid5, 0);
			menuBar()->setItemEnabled(Stm, 1);
			menuBar()->setItemEnabled(Obm, 1);
			ObjMenu->setItemEnabled(ShapeM, 1);
			StilMenu->clear();
			StilMenu->insertItem(tr("Color"), ColorMenu);
			StilMenu->insertItem(tr("Invert"));
			werkz->Textedit->setEnabled(true);
			werkz->Rotiere->setEnabled(true);
			break;
		case 4:
			fileMenu->changeItem(fid2, tr("Get Text..."));
			fileMenu->setItemEnabled(fid2, 1);
			fileMenu->setItemEnabled(fid3, 1);
			editMenu->setItemEnabled(edid1, 1);
			editMenu->setItemEnabled(edid2, 1);
			editMenu->setItemEnabled(edid4, 0);
			editMenu->setItemEnabled(edid5, 0);
			menuBar()->setItemEnabled(Stm, 1);
			menuBar()->setItemEnabled(Obm, 1);
			ObjMenu->setItemEnabled(ShapeM, 1);
			StilMenu->clear();
			StilMenu->insertItem(tr("Font"), FontMenu);
			StilMenu->insertItem(tr("Size"), SizeTMenu);
			StilMenu->insertItem(tr("Color"), ColorMenu);
			StilMenu->insertItem(tr("Shade"), ShadeMenu);
			werkz->Textedit->setEnabled(true);
			werkz->Rotiere->setEnabled(true);
			break;
		default:
			fileMenu->changeItem(fid2, tr("Get Text/Picture..."));
			fileMenu->setItemEnabled(fid2, 0);
			fileMenu->setItemEnabled(fid3, 0);
			editMenu->setItemEnabled(edid1, 1);
			editMenu->setItemEnabled(edid2, 1);
			editMenu->setItemEnabled(edid4, 0);
			editMenu->setItemEnabled(edid5, 0);
			menuBar()->setItemEnabled(Stm, 1);
			menuBar()->setItemEnabled(Obm, 1);
			ObjMenu->setItemEnabled(ShapeM, 0);
			StilMenu->clear();
			StilMenu->insertItem(tr("Color"), ColorMenu);
			StilMenu->insertItem(tr("Shade"), ShadeMenu);
			werkz->Textedit->setEnabled(false);
			werkz->Rotiere->setEnabled(true);
			break;
		}
	doc->CurrentSel = Nr;
	if (doc->ActPage->SelItem.count() > 1)
		{
		ObjMenu->setItemEnabled(DistM, 1);
		}
	else
		{
		ObjMenu->setItemEnabled(DistM, 0);
		}
}

void ScribusApp::slotDocCh()
{
	Tpal->BuildTree(view);
	if (!doc->isModified())
		{
		doc->setModified();
		setCaption(tr(doc->DocName) + "*");
		if (!doc->DocName.startsWith("Document "))
			{
			fileMenu->setItemEnabled(fid4, 1);
			}
		}
}

void ScribusApp::slotDocOpen()
{
	if (HaveDoc)
		{
		slotFileClose();
		}
	QString fileName = QFileDialog::getOpenFileName(0,tr("Documents (*.doc);; All Files (*)"),this);
	LadeDoc(fileName);
	}
	
void ScribusApp::LadeDoc(QString fileName)
{
	int a;
	QString DoFonts, tmp, tmp2;
	QFont fo;
  if (!fileName.isEmpty())
  	{
  	Serializer *ss = new Serializer(fileName);
  	if (ss->Read())
  		{
  		ss->GetDoc();
			doc=new ScribusDoc();
			HaveDoc = true;
			doc->Scale = 1.0;
			doc->AppMode = 1;
  		doc->PageColors = ss->GetColors();
			Cpal->SetColors(doc->PageColors);
			ss->GetInfo(doc);
			ss->GetValues(doc);
			DoFonts = ss->GetFonts();
			QTextStream t(&DoFonts, IO_ReadOnly);
			QMap<QString,QString> FontErs;
			FontErs.clear();
			while (!t.atEnd())
				{
				tmp = t.readLine();
				if (tmp != "")
					{
					if (!AvailFonts.contains(tmp))
						{
						DmF *dia = new DmF(this, tmp, AvailFonts);
						dia->exec();
						tmp2 = dia->Ersatz;
						delete dia;
						FontErs[tmp] = tmp2;
						tmp = tmp2;
						}
					fo = AvailFonts[tmp].Font;
					fo.setPointSize(doc->Dsize);
					doc->AddFont(tmp, fo);
					}
				}
			if (doc->UsedFonts.contains(DefFont))
				{
				doc->Dfont = DefFont;
				}
			else
				{
				doc->AddFont(DefFont, AvailFonts[DefFont].Font);
				doc->Dfont = DefFont;
				}
			doc->Dsize = DefSize;
			doc->minorGrid = DminGrid;
			doc->majorGrid = DmajGrid;
			doc->minorColor = DminColor;
			doc->majorColor = DmajColor;
			view=new ScribusView(this, doc);
			setCentralWidget(view);
			doc->loading = true;
			for (a=0; a<ss->Count; a++)
				{
				slotNewPage();
				}
			ss->GetPage(view, FontErs);
			doc->DocName = fileName;
			HaveNewDoc();
			doc->setUnModified();
			doc->loading = false;
			view->GotoPage(0);
  		}
  	delete ss;
  	}
}

void ScribusApp::slotFileOpen()
{
  if (doc->ActPage->SelItem.count() != 0)
  	{
 		PageItem *b = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0));
  	if (b->PType == 2)
  		{
  		QString fileName = QFileDialog::getOpenFileName(0,tr("Images (*.png *.gif *.jpg *.xpm);;Postscript (*.eps);;All Files (*)"),this);
  		if (!fileName.isEmpty())
  			{
    		doc->ActPage->LoadPict(fileName, *doc->ActPage->SelItem.at(0));
    		doc->ActPage->repaint();
    		QString message=tr("Picture ")+fileName+tr(" loaded");
    		statusBar()->message(message, 4000);
  			}
  		else
  			{
    		statusBar()->message(tr("Loading aborted"), 4000);
  			}
  		}
  	if (b->PType == 4)
  		{
  		QString fileName = QFileDialog::getOpenFileName(0,tr("Textfiles (*.txt);;All Files (*)"),this);
  		if (!fileName.isEmpty())
  			{
  			Serializer *ss = new Serializer(fileName);
  			if (ss->Read())
  				{
  				ss->GetText(b);
  				}
  			delete ss;
    		doc->ActPage->repaint();
    		QString message=tr("file ")+fileName+tr(" loaded");
    		statusBar()->message(message, 4000);
  			}
  		else
  			{
    		statusBar()->message(tr("Loading aborted"), 4000);
  			}
  		}
  	}
}


void ScribusApp::slotFileSave()
{
  QString fn = doc->DocName;
	Serializer *ss = new Serializer(fn);
	ss->PutDoc(doc);
	ss->PutPage(view);
	ss->Write();
	delete ss;
	doc->setUnModified();
	setCaption(fn);
	fileMenu->setItemEnabled(fid4, 0);
  statusBar()->message(tr("Ready."));
}

void ScribusApp::slotFileSaveAs()
{
  QString fn = QFileDialog::getSaveFileName(0, tr("Documents (*.doc);; All Files (*)"), this);
  if (!fn.isEmpty())
  	{
    doc->DocName = fn;
		Serializer *ss = new Serializer(fn);
		ss->PutDoc(doc);
		ss->PutPage(view);
		ss->Write();
		delete ss;
		doc->setUnModified();
		setCaption(fn);
		fileMenu->setItemEnabled(fid4, 0);
  	}
  else
  	{
    statusBar()->message(tr("Saving aborted"), 2000);
  	}
  statusBar()->message(tr("Ready"));
}

void ScribusApp::slotFileClose()
{
  uint a;
	if ((HaveDoc) && doc->isModified())
		{
		slotFileSaveAs();
		}
  view->close();
  for (a = 0; a<view->Pages.count(); a++)
  	{
  	delete view->Pages.at(a);
  	}
  delete view;
  delete doc;
  HaveDoc = FALSE;
	fileMenu->setItemEnabled(fid1, 0);
	fileMenu->setItemEnabled(fid2, 0);
	fileMenu->setItemEnabled(fid3, 0);
	fileMenu->setItemEnabled(fid4, 0);
	fileMenu->setItemEnabled(fid5, 0);
	fileMenu->setItemEnabled(fid6, 0);
	fileMenu->setItemEnabled(fid7, 0);
	fileMenu->setItemEnabled(fid8, 0);
	fileMenu->setItemEnabled(fid9, 0);
	fileMenu->setItemEnabled(fid10, 0);
	editMenu->setItemEnabled(edid1, 0);
	editMenu->setItemEnabled(edid2, 0);
	editMenu->setItemEnabled(edid3, 0);
	editMenu->setItemEnabled(edid4, 0);
	editMenu->setItemEnabled(edid5, 0);
	for (a=0; a<6; a++)
		{
		viewMenu->setItemEnabled(viewMenu->idAt(a), 0);
		}
	viewMenu->setItemEnabled(Markers, 0);
	viewMenu->setItemEnabled(Bilder, 0);
	viewMenu->setItemEnabled(Ras, 0);
	viewMenu->setItemEnabled(uRas, 0);
	viewMenu->setItemChecked(uRas, false);
	viewMenu->changeItem(Markers, tr("Hide Margins"));
	viewMenu->changeItem(Bilder, tr("Hide Images"));
	viewMenu->changeItem(Ras, tr("Show Guides"));
	menuBar()->setItemEnabled(pgmm, 0);
	menuBar()->setItemEnabled(Stm, 0);
	menuBar()->setItemEnabled(Obm, 0);
	ColorMenu->clear();
	Cpal->SetColors(DColors);
  statusBar()->message(tr("Ready."));
  setCaption(tr("Scribus " VERSION));
  Tpal->ListView1->clear();
}

void ScribusApp::slotFilePrint()
{
  QString fna, prn, cmd, cc, data;
  int Anf, Ende, Nr;
  bool fil;
  statusBar()->message(tr("Printing..."));
  QPrinter printer;
  printer.setMinMax(1, view->Pages.count());
  printer.setFromTo(1, view->Pages.count());
  if (printer.setup(this))
  	{
  	prn = printer.printerName();
  	fna = printer.outputFileName();
  	fil = printer.outputToFile();
  	Anf = printer.fromPage();
  	Ende = printer.toPage();
  	Nr = printer.numCopies();
		PSLib *dd = new PSLib(true, AvailFonts, doc->UsedFonts);
  	if (printer.pageOrder() == QPrinter::FirstPageFirst)
  		{
			data =  view->CreatePS(dd, Anf-1, Ende, 1);
			}
		else
			{
			data =  view->CreatePS(dd, Ende-1, Anf-2, -1);
			}
		if (fil)
			{
			QFile f(fna);	
			if (f.open(IO_WriteOnly))
				{
				QTextStream t(&f);
				t.writeRawBytes(data, data.length());
				f.close();	
				}
			}
		else
			{
			QFile f("/tmp/sc.ps");	
			if (f.open(IO_WriteOnly))
				{
				QTextStream t(&f);
				t.writeRawBytes(data, data.length());
				f.close();	
				}
			cmd = "lpr -P" + prn;
			if (Nr > 1)
				{
				cmd += " -#" + cc.setNum(Nr);
				}
			cmd += " /tmp/sc.ps";
			system(cmd);
			system("rm -f /tmp/sc.ps");
			}
		delete dd;
  	}
	statusBar()->message(tr("Ready"));
}

void ScribusApp::slotFileQuit()
{
  close();
}

void ScribusApp::slotEditCut()
{
	uint a;
	if ((HaveDoc) && (doc->ActPage->SelItem.count() != 0))
		{
		Buffer2 = "";
		PageItem *b = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0));
		if ((doc->AppMode == 7) && (b->HasSel))
			{
			for (a = 0; a < b->Ptext.count(); a++)
				{
				if (b->Ptext.at(a)->cselect)
					{
					Buffer2 += QString::number(b->Ptext.at(a)->ch)+"\t";
					Buffer2 += b->Ptext.at(a)->cfont+"\t";
					Buffer2 += QString::number(b->Ptext.at(a)->csize)+"\t";
					Buffer2 += b->Ptext.at(a)->ccolor+"\t";
					Buffer2 += QString::number(b->Ptext.at(a)->cextra)+"\t";
					Buffer2 += QString::number(b->Ptext.at(a)->cshade)+'\n';
					}
				}
			DeleteSel(b);
			b->Dirty = true;
			b->paintObj();
			BufferSet2 = true;
			}
		else
			{
			doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0))->CopyIt(&Buffer);
			BufferSet = true;
			doc->ActPage->DeleteItem();
			}
		slotDocCh();
		editMenu->setItemEnabled(edid3, 1);
		}
}

void ScribusApp::slotEditCopy()
{
	uint a;
	if ((HaveDoc) && (doc->ActPage->SelItem.count() != 0))
		{
		Buffer2 = "";
		PageItem *b = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0));
		if ((doc->AppMode == 7) && (b->HasSel))
			{
			for (a = 0; a < b->Ptext.count(); a++)
				{
				if (b->Ptext.at(a)->cselect)
					{
					Buffer2 += QString::number(b->Ptext.at(a)->ch)+"\t";
					Buffer2 += b->Ptext.at(a)->cfont+"\t";
					Buffer2 += QString::number(b->Ptext.at(a)->csize)+"\t";
					Buffer2 += b->Ptext.at(a)->ccolor+"\t";
					Buffer2 += QString::number(b->Ptext.at(a)->cextra)+"\t";
					Buffer2 += QString::number(b->Ptext.at(a)->cshade)+'\n';
					}
				}
			BufferSet2 = true;
			}
		else
			{
			doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0))->CopyIt(&Buffer);
			BufferSet = true;
			}
		editMenu->setItemEnabled(edid3, 1);
		}
}

void ScribusApp::slotEditPaste()
{
	struct Pti *hg;
	if (HaveDoc)
		{
		if ((doc->AppMode == 7) && (BufferSet2))
			{
			PageItem *b = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0));
			QTextStream t(&Buffer2, IO_ReadOnly);
			QString cc;
			while (!t.atEnd())
				{
				cc = t.readLine();
				QStringList wt;
				QStringList::Iterator it;
				wt = QStringList::split("\t", cc);
				it = wt.begin();
				hg = new Pti;
				hg->ch = (*it).toInt();
				it++;
				hg->cfont = *it;
				it++;
				hg->csize = (*it).toInt();
				it++;
				hg->ccolor = *it;
				it++;
				hg->cextra = (*it).toInt();
				it++;
				hg->cshade = (*it).toInt();
				hg->cselect = false;
 				b->Ptext.insert(b->CPos, hg);
 				b->CPos += 1;
 				}
 			if (b->CPos < int(b->Ptext.count())) { b->Dirty = true; }
 			b->paintObj();
			}
		else
			{
			if (BufferSet);
				{
				doc->ActPage->PasteItem(&Buffer, false);
				}	
			}
		slotDocCh();
		}
}

void ScribusApp::SelectAll()
{
	uint a;
	PageItem *b = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0));
	for (a = 0; a < b->Ptext.count(); a++)
		{
		b->Ptext.at(a)->cselect = true;
		}
	b->HasSel = true;
	b->Dirty = true;
	b->paintObj();
	EnableTxEdit();
}

void ScribusApp::DeleteText()
{
	PageItem *b = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0));
	DeleteSel(b);
	b->Dirty = true;
	b->paintObj();
	slotDocCh();
}

void ScribusApp::EnableTxEdit()
{
	editMenu->setItemEnabled(edid1, 1);
	editMenu->setItemEnabled(edid2, 1);
	editMenu->setItemEnabled(edid4, 1);
}

void ScribusApp::DisableTxEdit()
{
	editMenu->setItemEnabled(edid1, 0);
	editMenu->setItemEnabled(edid2, 0);
	editMenu->setItemEnabled(edid4, 0);
}

void ScribusApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About Scribus"),
                     "Scribus - 0.3\n\nAutor Franz Schmid\nE-Mail: Franz.Schmid@altmuehlnet.de" );
}

void ScribusApp::slotHelpAboutQt()
{
  QMessageBox::aboutQt(this,tr("About Qt"));
}

void ScribusApp::SaveText()
{
  QString fn = QFileDialog::getSaveFileName(0, tr("Textfiles (*.txt);;All Files (*)"), this);
  if (!fn.isEmpty())
  	{
    Serializer *se = new Serializer(fn);
    se->PutText(doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0)));
    se->Write();
    delete se;
  	}
  else
  	{
    statusBar()->message(tr("Saving aborted"), 2000);
  	}
}

/** Erzeugt eine neue Seite */
void ScribusApp::slotNewPage()
{
	view->addPage();
	if (view->Pages.count() > 1)
		{
		pageMenu->setItemEnabled(pgmd, 1);
		}
	connect(doc->ActPage, SIGNAL(PaintingDone()), werkz, SLOT(slotSelect()));
	connect(doc->ActPage, SIGNAL(HaveSel(int)), this, SLOT(HaveNewSel(int)));
	connect(doc->ActPage, SIGNAL(DocChanged()), this, SLOT(slotDocCh()));
	connect(doc->ActPage, SIGNAL(ItemPos(float, float)), Mpal, SLOT(setXY(float, float)));
	connect(doc->ActPage, SIGNAL(ItemGeom(float, float)), Mpal, SLOT(setBH(float, float)));
	connect(doc->ActPage, SIGNAL(SetAngle(float)), Mpal, SLOT(setR(float)));
	connect(doc->ActPage, SIGNAL(HaveSel(int)), Mpal, SLOT(NewSel(int)));
	connect(doc->ActPage, SIGNAL(SetSizeValue(int)), Mpal, SLOT(setSvalue(int)));
	connect(doc->ActPage, SIGNAL(SetLineArt(PenStyle)), Mpal, SLOT( setLIvalue(PenStyle)));
	connect(doc->ActPage, SIGNAL(ItemFarben(QString, QString, int, int)), this, SLOT(setCSMenu(QString, QString, int, int)));
	connect(doc->ActPage, SIGNAL(SetLocalValues(float, float, float, float)), Mpal, SLOT(setLvalue(float, float, float, float)));
	connect(doc->ActPage, SIGNAL(ItemFarben(QString, QString, int, int)), Cpal, SLOT(setActFarben(QString, QString, int, int)));
	connect(doc->ActPage, SIGNAL(ItemTextAttr(int)), Mpal, SLOT(SetTextVals(int)));
	connect(doc->ActPage, SIGNAL(ItemTextFont(QString)), this, SLOT(AdjustFontMenu(QString)));
	connect(doc->ActPage, SIGNAL(ItemTextFont(QString)), Mpal, SLOT(AdjustFMenu(QString)));
	connect(doc->ActPage, SIGNAL(ItemTextSize(int)), this, SLOT(setFSizeMenu(int)));
	connect(doc->ActPage, SIGNAL(ItemTextSize(int)), Mpal, SLOT(setFSizeMenu(int)));
	connect(doc->ActPage, SIGNAL(HasTextSel()), this, SLOT(EnableTxEdit()));
	connect(doc->ActPage, SIGNAL(HasNoTextSel()), this, SLOT(DisableTxEdit()));
	connect(doc->ActPage, SIGNAL(CopyItem()), this, SLOT(slotEditCopy()));
	connect(doc->ActPage, SIGNAL(CutItem()), this, SLOT(slotEditCut()));
	slotDocCh();
}

/** Ansicht absolut zoomen */
void ScribusApp::slotZoomAbs(float z)
{
	doc->Scale = z;
	view->slotDoZoom();
	statusBar()->message(tr("Scaled to: ")+QString::number(doc->Scale*100)+" %", 2000);
}

void ScribusApp::slotZoomFit()
{
	float dx = (view->width()-50) / (doc->PageB+30);
	float dy = (view->height()-70) / (doc->PageH+30);
	if (dx > dy)
		{
		slotZoomAbs(dy);
		}
	else
		{
		slotZoomAbs(dx);
		}
}

/** Ansicht 20 % */
void ScribusApp::slotZoom20()
{
	slotZoomAbs(0.2);
}

/** Ansicht 50 % */
void ScribusApp::slotZoom50()
{
	slotZoomAbs(0.5);
}

/** Ansicht 75 % */
void ScribusApp::slotZoom75()
{
	slotZoomAbs(0.75);
}

/** Ansicht 100 % */
void ScribusApp::slotZoom100()
{
	slotZoomAbs(1.0);
}

/** Ansicht 200 % */
void ScribusApp::slotZoom200()
{
	slotZoomAbs(2.0);
}

void ScribusApp::ToggleMarks()
{
	if (doc->Marks)
	{
		doc->Marks = FALSE;
		viewMenu->changeItem(Markers, tr("Show Margins"));
	}
	else
	{
		doc->Marks = TRUE;
		viewMenu->changeItem(Markers, tr("Hide Margins"));
	}
	view->DrawNew();
}

void ScribusApp::ToggleMpal()
{
	if (Mpal->isVisible())
	{
		Mpal->close();
		viewMenu->changeItem(viewMpal, tr("Show Measurements"));
	}
	else
	{
		Mpal->show();
		viewMenu->changeItem(viewMpal, tr("Hide Measurements"));
	}
}

void ScribusApp::ToggleTpal()
{
	if (Tpal->isVisible())
	{
		Tpal->close();
		viewMenu->changeItem(viewTpal, tr("Show Outline"));
	}
	else
	{
		if (HaveDoc)
			{
			Tpal->BuildTree(view);
			}
		Tpal->show();
		viewMenu->changeItem(viewTpal, tr("Hide Outline"));
	}
}

void ScribusApp::ToggleCpal()
{
	if (Cpal->isVisible())
	{
		Cpal->close();
		viewMenu->changeItem(viewCpal, tr("Show Colors"));
	}
	else
	{
		Cpal->show();
		viewMenu->changeItem(viewCpal, tr("Hide Colors"));
	}
}

void ScribusApp::ToggleTools()
{
	if (werkz->isVisible())
	{
		werkz->close();
		viewMenu->changeItem(viewTools, tr("Show Tools"));
	}
	else
	{
		werkz->show();
		viewMenu->changeItem(viewTools, tr("Hide Tools"));
	}
}

void ScribusApp::TogglePics()
{
	uint a, b;
	if (doc->ShowPic)
		{
		viewMenu->changeItem(Bilder, tr("Show Images"));
		doc->ShowPic = FALSE;
		}
	else
		{
		viewMenu->changeItem(Bilder, tr("Hide Images"));
		doc->ShowPic = TRUE;
		}
	for (a=0; a<view->Pages.count(); a++)
		{
		for (b=0; b<view->Pages.at(a)->Items.count(); b++)
			{
			if (view->Pages.at(a)->Items.at(b)->PType == 2)
				{
				view->Pages.at(a)->Items.at(b)->PicArt = doc->ShowPic;
				}
			}
		view->Pages.at(a)->repaint();
		}
}

void ScribusApp::ToggleRaster()
{
	uint a;
	if (doc->Raster)
		{
		viewMenu->changeItem(Ras, tr("Show Guides"));
		doc->Raster = FALSE;
		}
	else
		{
		viewMenu->changeItem(Ras, tr("Hide Guides"));
		doc->Raster = TRUE;
		}
	for (a=0; a<view->Pages.count(); a++)
		{
		view->Pages.at(a)->repaint();
		}
}

void ScribusApp::ToggleURaster()
{
	if (doc->useRaster)
		{
		doc->useRaster = FALSE;
		}
	else
		{
		doc->useRaster = TRUE;
		}
	viewMenu->setItemChecked(uRas, doc->useRaster);
}

void ScribusApp::ToggleFrameEdit()
{
	if (doc->EditClip)
		{
		doc->EditClip = FALSE;
		}
	else
		{
		doc->EditClip = TRUE;
		}
	ShapeMenu->setItemChecked(ShapeEdit, doc->EditClip);
}

void ScribusApp::setAppMode(int mode)
{
	setActiveWindow();
	if (HaveDoc)
		{
		int oldMode = doc->AppMode;
		doc->AppMode = mode;
		if (oldMode == 7)
			{
			disconnect(CurTimer, SIGNAL(timeout()), doc->ActPage, SLOT(BlinkCurs()));
			CurTimer->stop();
			if (BufferSet)
				{
				editMenu->setItemEnabled(edid3, 1);
				}
			else
				{
				editMenu->setItemEnabled(edid3, 0);
				}
			editMenu->setItemEnabled(edid4, 0);
			editMenu->setItemEnabled(edid5, 0);
			doc->ActPage->slotDoCurs(false);
			doc->ActPage->repaint();
			}
		if (mode == 7)
			{
			doc->ActPage->slotDoCurs(true);
			CurTimer = new QTimer(doc->ActPage);
			connect(CurTimer, SIGNAL(timeout()), doc->ActPage, SLOT(BlinkCurs()));
			CurTimer->start(500);
			if (doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0))->HasSel)
				{
				editMenu->setItemEnabled(edid1, 1);
				editMenu->setItemEnabled(edid2, 1);
				editMenu->setItemEnabled(edid4, 1);
				editMenu->setItemEnabled(edid5, 1);
				}
			else
				{
				editMenu->setItemEnabled(edid1, 0);
				editMenu->setItemEnabled(edid2, 0);
				editMenu->setItemEnabled(edid4, 0);
				editMenu->setItemEnabled(edid5, 1);
				}
			if (BufferSet2)
				{
				editMenu->setItemEnabled(edid3, 1);
				}
			else
				{
				editMenu->setItemEnabled(edid3, 0);
				}
			doc->ActPage->repaint();
			}
		if (mode == 6)
			{
			qApp->setOverrideCursor(QCursor(QPixmap(Lupe)), 1);
			}
		else
			{
			qApp->setOverrideCursor(QCursor(ArrowCursor), 1);
			}
		}
}

void ScribusApp::setItemXp()
{
	float x, y;
	int yi;
	bool ok;
	QString xv, yv, bv, hv, rv, lv;
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		xv = Mpal->X_val->text();
		yv = Mpal->Y_val->text();
		bv = Mpal->B_val->text();
		hv = Mpal->H_val->text();
		rv = Mpal->R_val->text();
		lv = Mpal->L_val->text();
		y = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0))->Xpos;
		x = xv.toFloat(&ok);
		if (ok)
			{
			doc->ActPage->MoveItem(x-y, 0, *doc->ActPage->SelItem.at(0));
			}
		y = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0))->Ypos;
		x = yv.toFloat(&ok);
		if (ok)
			{
			doc->ActPage->MoveItem(0, x-y, *doc->ActPage->SelItem.at(0));
			}
		y = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0))->Height;
		x = bv.toFloat(&ok);
		if (ok)
			{
			doc->ActPage->SizeItem(x, y, *doc->ActPage->SelItem.at(0));
			}
		y = doc->ActPage->Items.at(*doc->ActPage->SelItem.at(0))->Width;
		x = hv.toFloat(&ok);
		if (ok)
			{
			doc->ActPage->SizeItem(y, x, *doc->ActPage->SelItem.at(0));
			}
		y = rv.toFloat(&ok) * (-1);
		if (ok)
			{
			doc->ActPage->RotateItem(y, *doc->ActPage->SelItem.at(0));
			}
		yi = lv.toInt(&ok);
		if (ok)
			{
			doc->ActPage->ChLineSpa(yi);
			}
		slotDocCh();
		}
}

void ScribusApp::setItemLspS(int w)
{
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		doc->ActPage->ChLineSpa(w);
		slotDocCh();
		}
}

void ScribusApp::setItemLineWidth(int w)
{
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		doc->ActPage->ChLineWidth(w);
		slotDocCh();
		}
}

void ScribusApp::setLineStipple(PenStyle w)
{
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		doc->ActPage->ChLineArt(w);
		slotDocCh();
		}
}

void ScribusApp::setItemLocalXY(float x, float y)
{
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		doc->ActPage->ChLocalXY(x, y);
		slotDocCh();
		}
}

void ScribusApp::setItemLocalSc(float x, float y)
{
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		doc->ActPage->ChLocalSc(x, y);
		slotDocCh();
		}
}

void ScribusApp::setImageV()
{
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		doc->ActPage->FlipImageV();
		slotDocCh();
		}
}

void ScribusApp::setImageH()
{
	setActiveWindow();
	if (doc->ActPage->SelItem.count() != 0)
		{
		doc->ActPage->FlipImageH();
		slotDocCh();
		}
}

void ScribusApp::DeletePage()
{
	view->delPage(doc->ActPage->PageNr);
	if (view->Pages.count() == 1)
		{
		pageMenu->setItemEnabled(pgmd, 0);
		slotDocCh();
		}
}

void ScribusApp::setItemFont(int id)
{
	QString nf;
	nf = *FontID[long(id)];
	SetNewFont(nf);
}

void ScribusApp::SetNewFont(QString nf)
{
	setActiveWindow();
	int a;
	if (!doc->UsedFonts.contains(nf))
		{
		doc->AddFont(nf, AvailFonts[nf].Font);
		SCFonts::Iterator it;
		for (it = AvailFonts.begin(); it != AvailFonts.end(); ++it)
			{
			if (it.key() == nf) break;
			}
		a = FontMenu->insertItem(new FmItem(it.key(), AvailFonts[nf].Font));
		FontID.insert(long(a), &it.key());
		a = Mpal->FontMenu->insertItem(new FmItem(it.key(), AvailFonts[nf].Font));
		Mpal->FontID.insert(long(a), &it.key());
		}
	AdjustFontMenu(nf);
	Mpal->AdjustFMenu(nf);
	doc->ActPage->ItemFont(nf);
	slotDocCh();
}

void ScribusApp::AdjustFontMenu(QString nf)
{
	uint a;
	QString df;
	for (a = 2; a < FontMenu->count(); a++)
		{
		df = *FontID[long(FontMenu->idAt(a))];
		if (df == nf)
			{
			FontMenu->setItemChecked(FontMenu->idAt(a), true);
			}
		else
			{
			FontMenu->setItemChecked(FontMenu->idAt(a), false);
			}
		}
}

void ScribusApp::setItemFSize(int id)
{
	int c = SizeTMenu->indexOf(id);
	bool ok = false;
	if (c > 0)
		{		
		doc->ActPage->chFSize(SizeTMenu->text(id).left(2).toInt());
		}
	else
		{
    Query* dia = new Query(this, "New", 1, 0, "Size:", "Size");
    if (dia->exec())
    	{
			c = dia->Answer->text().toInt(&ok);
			if (ok)
				{	
				doc->ActPage->chFSize(c);
				}
			delete dia;
     	}
		}
	slotDocCh();
}

void ScribusApp::setFSizeMenu(int size)
{
	uint a;
	for (a = 0; a < SizeTMenu->count(); a++)
		{
		SizeTMenu->setItemChecked(SizeTMenu->idAt(a), false);
		if (SizeTMenu->text(SizeTMenu->idAt(a)).left(2).toInt() == size)
			{
			SizeTMenu->setItemChecked(SizeTMenu->idAt(a), true);
			}
		}
}

void ScribusApp::setFontSize(int size)
{
	setActiveWindow();
	doc->ActPage->chFSize(size);
	slotDocCh();
}

void ScribusApp::setItemFarbe(int id)
{
	uint a;
	for (a = 0; a < ColorMenu->count(); a++)
		{
		ColorMenu->setItemChecked(ColorMenu->idAt(a), false);
		}
	ColorMenu->setItemChecked(id, true);
	doc->ActPage->ItemBrush(ColorMenu->text(id));
	slotDocCh();
}

void ScribusApp::setItemShade(int id)
{
	int c = ShadeMenu->indexOf(id);
	uint a;
	bool ok = false;
	for (a = 0; a < ShadeMenu->count(); a++)
		{
		ShadeMenu->setItemChecked(ShadeMenu->idAt(a), false);
		}
	ShadeMenu->setItemChecked(id, true);
	if (c > 0)
		{		
		doc->ActPage->ItemBrushShade((c-1) * 10);
		}
	else
		{
    Query* dia = new Query(this, "New", 1, 0, "Shade:", "Shade");
    if (dia->exec())
    	{
			c = dia->Answer->text().toInt(&ok);
			if (ok)
				{		
				doc->ActPage->ItemBrushShade(c);
				}
			delete dia;
     	}
		}
	slotDocCh();
}

void ScribusApp::setCSMenu(QString , QString l, int , int ls)
{
	uint a;
	for (a = 0; a < ColorMenu->count(); a++)
		{
		ColorMenu->setItemChecked(ColorMenu->idAt(a), false);
		if (ColorMenu->text(ColorMenu->idAt(a)) == l)
			ColorMenu->setItemChecked(ColorMenu->idAt(a), true);
		}
	for (a = 0; a < ShadeMenu->count(); a++)
		{
		ShadeMenu->setItemChecked(ShadeMenu->idAt(a), false);
		}
	ShadeMenu->setItemChecked(ShadeMenu->idAt(ls/10+1), true);
}

void ScribusApp::slotEditColors()
{
	int a;
	uint b, c, d;
	CListe edc;
	QMap<QString,QString> ers;
	if (HaveDoc)
		{
		edc = doc->PageColors;
		}
	else
		{
		edc = DColors;
		}
	Farbmanager* dia = new Farbmanager(this, edc);
	if (dia->exec())
		{
		if (HaveDoc)
			{
			slotDocCh();
			doc->PageColors = dia->EditColors;
			Cpal->SetColors(doc->PageColors);
			CListe::Iterator it;
			ColorMenu->clear();
			QPixmap pm = QPixmap(15, 15);
			a = 0;
			for (it = doc->PageColors.begin(); it != doc->PageColors.end(); ++it)
				{
				pm.fill(doc->PageColors[it.key()]);
				ColorMenu->insertItem(QIconSet(pm), it.key());
				if (it.key() == doc->Dbrush)
					{
					ColorMenu->setItemChecked(ColorMenu->idAt(a), true);
					}
				a++;
				}
			ers = dia->Ersatzliste;
			if (!ers.isEmpty())
				{
				for (it = doc->PageColors.begin(); it != doc->PageColors.end(); ++it)
					{
					ers.remove(it.key());
					}
				if (!ers.isEmpty())
					{
					QMap<QString,QString>::Iterator it;
					for (it = ers.begin(); it != ers.end(); ++it)
						{
						for (b=0; b<view->Pages.count(); b++)
							{
							for (c=0; c<view->Pages.at(b)->Items.count(); c++)
								{
								if (view->Pages.at(b)->Items.at(c)->PType == 4)
									{
									for (d=0; d<view->Pages.at(b)->Items.at(c)->Ptext.count(); d++)
										{
										if (it.key() == view->Pages.at(b)->Items.at(c)->Ptext.at(d)->ccolor)
											{
											view->Pages.at(b)->Items.at(c)->Ptext.at(d)->ccolor = it.data();
											}
										}
									}
								if (it.key() == view->Pages.at(b)->Items.at(c)->Pcolor)
									{
									view->Pages.at(b)->Items.at(c)->Pcolor = it.data();
									}
								if (it.key() == view->Pages.at(b)->Items.at(c)->Pcolor2)
									{
									view->Pages.at(b)->Items.at(c)->Pcolor2 = it.data();
									}
								}
							}
						}
					}
				}
			view->DrawNew();
			}
		else
			{
			DColors = dia->EditColors;
			Cpal->SetColors(DColors);
			}
		}
	delete dia;
}

void ScribusApp::setPenFarbe(QString farbe)
{
	setActiveWindow();
	if (HaveDoc)
		{
		doc->ActPage->ItemPen(farbe);
		slotDocCh();
		}
}

void ScribusApp::setPenShade(int sh)
{
	setActiveWindow();
	if (HaveDoc)
		{
		doc->ActPage->ItemPenShade(sh);
		slotDocCh();
		}
}

void ScribusApp::setBrushFarbe(QString farbe)
{
	setActiveWindow();
	if (HaveDoc)
		{
		doc->ActPage->ItemBrush(farbe);
		slotDocCh();
		}
}

void ScribusApp::setBrushShade(int sh)
{
	setActiveWindow();
	if (HaveDoc)
		{
		doc->ActPage->ItemBrushShade(sh);
		slotDocCh();
		}
}

void ScribusApp::GetBrushPen()
{
	setActiveWindow();
	if (HaveDoc)
		{
		doc->ActPage->QueryFarben();
		slotDocCh();
		}
}

void ScribusApp::RectFrame()
{
	doc->ActPage->SetFrameRect();
	slotDocCh();
}

void ScribusApp::OvalFrame()
{
	doc->ActPage->SetFrameOval();
	slotDocCh();
}

void ScribusApp::DeleteObjekt()
{
	doc->ActPage->DeleteItem();
}

void ScribusApp::Objekt2Back()
{
	doc->ActPage->ToBack();
}

void ScribusApp::Objekt2Front()
{
	doc->ActPage->ToFront();
}

void ScribusApp::ObjektRaise()
{
	doc->ActPage->RaiseItem();
}

void ScribusApp::ObjektLower()
{
	doc->ActPage->LowerItem();
}

void ScribusApp::ObjektDup()
{
	slotEditCopy();
	slotEditPaste();
	doc->ActPage->MoveItem(DispX, DispY, *doc->ActPage->SelItem.at(0));
}

void ScribusApp::ObjektDupM()
{
	Mdup *dia = new Mdup(this, DispX, DispY);
	if (dia->exec())
		{
		int anz = dia->Ncopies->text().toInt();
		float dH = dia->ShiftH->text().toFloat();
		float dV = dia->ShiftV->text().toFloat();
		int a;
		if (anz>0)
			{
			for (a=0; a<anz; a++)
				{
				slotEditCopy();
				slotEditPaste();
				doc->ActPage->MoveItem(dH, dV, *doc->ActPage->SelItem.at(0));
				DispX = dH;
				DispY = dV;
				}
			slotDocCh();
			}
		}
	delete dia;
}

void ScribusApp::SelectFromOutl(int Page, int Item)
{
	setActiveWindow();
	doc->ActPage->Deselect();
	view->GotoPage(Page);
	doc->ActPage->SelectItemNr(Item);
}

void ScribusApp::SelectFromOutlS(int Page)
{
	setActiveWindow();
	doc->ActPage->Deselect();
	view->GotoPage(Page);
}

void ScribusApp::InfoDoc()
{
	DocInfos *dia = new DocInfos(this, doc->DocAutor, doc->DocTitel, doc->DocComments);
	if (dia->exec())
		{
		doc->DocAutor = dia->AutorInfo->text();
		doc->DocTitel = dia->TitelInfo->text();
		doc->DocComments = dia->MultiLineEdit1->text();
		slotDocCh();
		}
	delete dia;
}

void ScribusApp::ObjektAlign()
{
	float xdp, ydp;
	bool xa, ya, Vth, Vtv;
	int xart, yart;
	Align *dia = new Align(this, doc->ActPage->SelItem.count());
	connect(dia, SIGNAL(ApplyDist(bool, bool, bool, bool, float, float, int, int)),
	        this, SLOT(DoAlign(bool, bool, bool, bool, float, float, int, int)));
	if (dia->exec())
		{
		xdp = float(dia->AHor->value());
		xa = dia->CheckH->isChecked();
		ydp = float(dia->AVert->value());
		ya = dia->CheckV->isChecked();
		xart = dia->VartH->currentItem();
		yart = dia->VartV->currentItem();
		Vth = dia->VerteilenH->isChecked();
		Vtv = dia->VerteilenV->isChecked();
		doc->ActPage->AlignObj(xa, ya, Vth, Vtv, xdp, ydp, xart, yart);
		slotDocCh();
		}
	delete dia;
}

void ScribusApp::DoAlign(bool xa, bool ya, bool Vth, bool Vtv, float xdp, float ydp, int xart, int yart)
{
	doc->ActPage->AlignObj(xa, ya, Vth, Vtv, xdp, ydp, xart, yart);
	slotDocCh();
}

void ScribusApp::BuildFontMenu()
{
	QMap<QString,int> Fami;
	QPopupMenu *pm;
	SCFonts::Iterator it;
	FontID.clear();
	FontMenu->clear();
	FStyleMenu->clear();
	int a;
	QString b = " ";
	for (it = AvailFonts.begin(); it != AvailFonts.end(); ++it)
		{
		Fami.insert(it.key().left(it.key().find(" ")), 0);
		}
	QMap<QString,int>::Iterator it2;
	it = AvailFonts.begin();
	for (it2 = Fami.begin(); it2 != Fami.end(); ++it2)
		{
		pm = new QPopupMenu();
		while (it2.key() == it.key().left(it.key().find(" ")))
			{
			a = pm->insertItem(new FmItem(it.key(), AvailFonts[it.key()].Font));
			FontID.insert(long(a), &it.key());
			it++;
			}
  	connect(pm, SIGNAL(activated(int)), this, SLOT(setItemFont(int)));
		FStyleMenu->insertItem(it2.key(), pm);
		}
	a = FontMenu->insertItem(tr("Style"), FStyleMenu);
	FontID.insert(long(a), &b);
	a = FontMenu->insertSeparator();
	FontID.insert(long(a), &b);
	if (!HaveDoc)
		{
		a = FontMenu->insertItem(new FmItem(AvailFonts.begin().key(), AvailFonts.begin().data().Font));
		FontMenu->setItemChecked(a, true);
		FontID.insert(long(a), &AvailFonts.begin().key());
		}
	else
		{
		QMap<QString,QFont>::Iterator it3;
		for (it3 = doc->UsedFonts.begin(); it3 != doc->UsedFonts.end(); ++it3)
			{
			a = FontMenu->insertItem(new FmItem(it3.key(), it3.data()));
			if (it3.key() == doc->Dfont)
				{
				FontMenu->setItemChecked(a, true);
				}
			FontID.insert(long(a), &it3.key());
			}
		}
}

void ScribusApp::GetAllFonts()
{
	struct Foi t;
	QString FDatei, tmp, Datei, Name, Fname, Stil, Extra, SCName, Pfad, tmp2, tmp6, tmp7, dummy, rn;
	QStringList wt, wt2;
	int fanz, a, b, bd;
	uint c;
	bool found = false;
	AvailFonts.clear();
	FILE *fp = popen("xset -q", "r");
	if (fp == NULL) return;
	QTextStream ts3(fp, IO_ReadOnly);
	tmp = ts3.read();
	QTextStream ts4(&tmp, IO_ReadOnly);
	while (!ts4.atEnd())
		{
		tmp2 = ts4.readLine();
		if (tmp2.startsWith("Font Path:"))
			{
			found = true;
			tmp2 = ts4.readLine();
			break;
			}
		}
	if (found)
		{
		if (!tmp2.contains("Type1"))
			{
			QString mess = tr("There are no Postscript-Fonts on your System");
			mess += "\n" + tr("Exiting now");
			QMessageBox::critical(this, tr("Fatal Error"), mess, 1, 0, 0);
			NoFonts = true;
			}
		wt2 = QStringList::split(",", tmp2);
		for (c = 0; c < wt2.count(); c++)
			{
			Pfad = wt2[c];
			if (Pfad.contains("Type1"))
				{
				if (Pfad.right(1) != "/")
					{
					Pfad += "/";
					}
				QFile f(Pfad+"fonts.dir");
				if (f.open(IO_ReadOnly))
					{
					QTextStream ts(&f);
					FDatei = ts.read();
					f.close();
					QTextStream ts2(&FDatei, IO_ReadOnly);
					fanz = ts2.readLine().toInt();
					for (a=0; a<fanz; a++)
						{
						tmp = ts2.readLine();
						b = tmp.find(".");
						Datei = tmp.left(b+4);
						Name = tmp.mid(b+5, tmp.length());
						wt = QStringList::split("-", Name);
						Fname = wt[1];
						Stil = wt[2];
						Extra = wt[3];
						SCName = Fname[0].upper() + Fname.remove(0, 1);
						SCName += " ";
						SCName += Stil[0].upper() + Stil.remove(0, 1);
						if (Extra == "i") SCName += " Italic";
						if (Extra == "o") SCName += " Oblique";
						t.Font = QFont();
						t.Font.setRawName(Name);
						t.Font.setPointSize(12);
						QFile fi(Pfad+Datei);
						tmp6 = "";
						tmp7 = "a /NoName";
						if (fi.open(IO_ReadOnly))
							{
							QTextStream ts5(&fi);
							tmp6 = ts5.read();
							fi.close();
							bd = tmp6.find("/FontName");
							tmp7 = tmp6.mid(bd, 80);
							}
						QTextIStream (&tmp7) >> dummy >> rn;	
						t.RealName = rn.remove(0, 1);
						t.Datei = Pfad+Datei;
						t.EmbedPS = true;
  					AvailFonts[SCName] = t;
						}
					}
				}
			}
		}
	pclose(fp);
}

void ScribusApp::slotFontOrg()
{
	uint a;
	FontPrefs *dia = new FontPrefs(this, AvailFonts);
	if (dia->exec())
		{
		a = 0;
		SCFonts::Iterator it;
		for (it = AvailFonts.begin(); it != AvailFonts.end(); ++it)
			{
			it.data().EmbedPS = dia->FlagsPS.at(a)->isChecked();
			a++;
			}
		}
	delete dia;
}

void ScribusApp::slotPrefsOrg()
{
	Preferences *dia = new Preferences(this,
																		 AvailFonts,
																		 DefFont,
																		 DefSize,
																		 Wheelval,
																		 DminGrid,
																		 DmajGrid,
																		 DminColor,
																		 DmajColor,
																		 GUI);
	if (dia->exec())
		{
		DefFont = dia->FontCombo->currentText();
		DefSize = dia->SizeCombo->currentText().left(2).toInt();
		Wheelval = dia->SpinBox3->value();
		DminGrid = dia->SpinBox1->value();
		DmajGrid = dia->SpinBox2->value();
		DminColor = dia->minColor->backgroundColor();
		DmajColor = dia->MaxColor->backgroundColor();
		if (dia->RadioButton1->isChecked())
			{
			GUI = 1;
      qApp->setStyle(new QPlatinumStyle);
			}
		if (dia->RadioButton2->isChecked())
			{
			GUI = 2;
      qApp->setStyle(new QWindowsStyle);
			}
		if (dia->RadioButton3->isChecked())
			{
			GUI = 3;
      qApp->setStyle(new QMotifStyle);
			}
		if (dia->RadioButton4->isChecked())
			{
			GUI = 4;
      qApp->setStyle(new QCDEStyle);
			}
		if (dia->RadioButton5->isChecked())
			{
			GUI = 5;
      qApp->setStyle(new QMotifPlusStyle);
			}
		if (HaveDoc)
			{
			doc->Dfont = DefFont;
			doc->Dsize = DefSize;
			doc->minorGrid = DminGrid;
			doc->majorGrid = DmajGrid;
			doc->minorColor = DminColor;
			doc->majorColor = DmajColor;
			view->DrawNew();
			}
		SavePrefs();
		}
	delete dia;
}

void ScribusApp::SavePrefs()
{
	QString ho = getenv("HOME");
	QFile f(ho+"/.scribus.rc");
	if (f.open(IO_WriteOnly))
		{
		QDataStream t(&f);
		t << GUI << DefFont << DefSize << DminGrid << DmajGrid << DminColor << DmajColor;
		t << int(werkz->isVisible()) << werkz->pos().x() << werkz->pos().y();
		t << int(Mpal->isVisible()) << Mpal->pos().x() << Mpal->pos().y();
		t << int(Cpal->isVisible()) << Cpal->pos().x() << Cpal->pos().y();
		t << int(Tpal->isVisible()) << Tpal->pos().x() << Tpal->pos().y();
		t << pos().x() << pos().y() << size().width() << size().height();
		SCFonts::Iterator it;
		QMap<QString,int> ff;
		ff.clear();
		for (it = AvailFonts.begin(); it != AvailFonts.end(); ++it)
			{
			ff[it.key()] = int(it.data().EmbedPS);
			}
		t << ff;
		t << DColors;
		f.close();
		}
}

void ScribusApp::ReadPrefs()
{
	QMap<QString,int> ff;
	ff.clear();
	QString ho = getenv("HOME");
	QFile f(ho+"/.scribus.rc");
	if (f.open(IO_ReadOnly))
		{
		QDataStream t(&f);
		int g, x, y, w, h;
		QString fg;
		t >> GUI >> DefFont >> DefSize >> DminGrid >> DmajGrid >> DminColor >> DmajColor;
		t >> g >> x >> y;
		if (bool(g))
			{
			Wsh = true;
			werkz->move(x, y);
			viewMenu->changeItem(viewTools, tr("Hide Tools"));
			}
		t >> g >> x >> y;
		if (bool(g))
			{
			Msh = true;
			Mpal->move(x, y);
			viewMenu->changeItem(viewMpal, tr("Hide Measurements"));
			}
		t >> g >> x >> y;
		if (bool(g))
			{
			Csh = true;
			Cpal->move(x, y);
			viewMenu->changeItem(viewCpal, tr("Hide Colors"));
			}
		t >> g >> x >> y;
		if (bool(g))
			{
			Tsh = true;
			Tpal->move(x, y);
			viewMenu->changeItem(viewTpal, tr("Hide Outline"));
			}
		t >> x >> y >> w >> h;
		move(x, y);
		resize(w, h);
		t >> ff;
		QMap<QString,int>::Iterator it;
		for (it = ff.begin(); it != ff.end(); ++it)
			{
			if (AvailFonts.contains(it.key()))
				{
				AvailFonts[it.key()].EmbedPS = bool(it.data());
				}
			}
		t >> DColors;
		switch (GUI)
			{
			case 1:
      	qApp->setStyle(new QPlatinumStyle);
      	break;
      case 2:
      	qApp->setStyle(new QWindowsStyle);
      	break;
      case 3:
      	qApp->setStyle(new QMotifStyle);
      	break;
      case 4:
      	qApp->setStyle(new QCDEStyle);
      	break;
      case 5:
      	qApp->setStyle(new QMotifPlusStyle);
      	break;
			}
		f.close();
		}
}

void ScribusApp::ShowSubs()
{
	QString mess;
	if ((HaveGS != 0) || (HaveConvert != 0) || (HavePFB > 256))
		{
		mess = tr("The following Programs are missing:")+"\n\n";
		if (HaveGS != 0)
			{
			mess += tr("Ghostscript : You cannot use EPS-Images")+"\n\n";
			PDFavailable = false;
			}
		if (HaveConvert != 0)
			mess += tr("convert : You cannot print Raster-Images")+"\n\n";
		if (HavePFB > 256)
			{
			SCFonts::Iterator it;
			for (it = AvailFonts.begin(); it != AvailFonts.end(); ++it)
				{
				if (it.data().Datei.right(1) == "b")
					it.data().EmbedPS = false;
				}
			mess += tr("pfb2pfa : You cannot embed .pfb-Fonts");
			}
    QMessageBox::warning(this, tr("Warning"), mess, 1, 0, 0);
    }
	if (Wsh)
		werkz->show();
	if (Msh)
		Mpal->show();
	if (Csh)
		Cpal->show();
	if (Tsh)
		Tpal->show();
	setActiveWindow();
	raise();
}

void ScribusApp::SaveAsEps()
{
  QString fn = QFileDialog::getSaveFileName(0, tr("EPS-Files (*.eps);; All Files (*)"), this);
  if (!fn.isEmpty())
  	{
		PSLib *dd = new PSLib(false, AvailFonts, doc->UsedFonts);
		QString data =  view->CreatePS(dd, doc->ActPage->PageNr, doc->ActPage->PageNr+1, 1);
		QFile f(fn);	
		if (f.open(IO_WriteOnly))
			{
			QTextStream t(&f);
			t.writeRawBytes(data, data.length());
			f.close();	
			}
		delete dd;
  	}
}

void ScribusApp::SaveAsPDF()
{
  QString fn;
  PDF_Opts *dia = new PDF_Opts(this, doc->DocName);
  if (dia->exec())
  	{
  	fn = dia->Datei->text();
		PSLib *dd = new PSLib(true, AvailFonts, doc->UsedFonts);
		QString data =  view->CreatePS(dd, 0, view->Pages.count(), 1);
		QFile f("/tmp/sc.ps");	
		if (f.open(IO_WriteOnly))
			{
			QTextStream t(&f);
			t.writeRawBytes(data, data.length());
			f.close();	
			}
		system("ps2pdf -dAutoRotatePages=/None "+dia->Opts+" /tmp/sc.ps "+fn);
		system("rm -f /tmp/sc.ps");
		delete dd;
  	}
  delete dia;
}
	
/** No descriptions
void ScribusApp::slotTest()
{
	doc->ActPage->PaintEllipse(10, 10, 100, 120, 3, "Blue", "Yellow");
	int z = doc->ActPage->PaintRect(40, 70, 100, 120, 1, "Red", "Green");
	doc->ActPage->Items.at(z)->Rot = -45;
	doc->ActPage->repaint();
	doc->ActPage->PaintLine(40, 170, 100, 120, 4, "Blue");
}

void ScribusApp::slotTest2()
{
	struct Pti *hg;
	uint a;
	QString Text = "Das ist ein Test";
	int z = doc->ActPage->PaintText(20, 20, 200, 100, 1, "Black");
	for (a=0; a<Text.length(); a++)
		{
		hg = new Pti;
		hg->ch = Text.at(a).unicode();
		hg->cfont = doc->Dfont;
		hg->csize = 12;
		hg->ccolor = "Magenta";
		hg->cextra = 0;
		hg->cshade = 100;
		hg->cselect = false;
		doc->ActPage->Items.at(z)->Ptext.append(hg);
		}
	doc->ActPage->Items.at(z)->LineSp = 12;
	doc->ActPage->Items.at(z)->isBookmark = true;		
	doc->ActPage->Items.at(z)->paintObj();
}

void ScribusApp::slotTest3()
{
}

void ScribusApp::slotTest4()
{
}   */

