/***************************************************************************
                          scribus.h  -  description
                             -------------------
    begin                : Fre Apr  6 21:09:31 CEST 2001
    copyright            : (C) 2001 by Franz Schmid
    email                : Franz.Schmid@altmuehlnet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCRIBUS_H
#define SCRIBUS_H

// include files for QT
#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qmap.h>
#include <qfont.h>
#include <qtimer.h>
#include <qintdict.h>

// application specific includes
#include "scribusview.h"
#include "scribusdoc.h"
#include "tools.h"
#include "mpalette.h"
#include "colorm.h"
#include "cpalette.h"
#include "tree.h"

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class ScribusView is
  * created which creates your view.
  */
class ScribusApp : public QMainWindow
{
  Q_OBJECT
  
  public:
    /** construtor */
    ScribusApp();
    /** destructor */
    ~ScribusApp();
    /** initMenuBar creates the menu_bar and inserts the menuitems */
    void initMenuBar();
    /** setup the statusbar */
    void initStatusBar();
    /** overloaded for Message box on last window exit */
    bool queryExit();
    void closeEvent(QCloseEvent *ce);
    void keyPressEvent(QKeyEvent *k);
    void wheelEvent(QWheelEvent *w);
    void DeleteSel(PageItem *b);
    void setTBvals(PageItem *b);
    void SavePrefs();
    void ReadPrefs();
    void ShowSubs();
    SCFonts AvailFonts;
    QString DefFont;
    int DefSize;
    CListe DColors;
    struct CLBuf Buffer;
    QString Buffer2;
    float DispX;
    float DispY;
    int Wheelval;
  	int DminGrid;
  	int DmajGrid;
  	QColor DminColor;
  	QColor DmajColor;
  	int GUI;
    QTimer *CurTimer;
    bool NoFonts;

  public slots:

    /** generate a new document in the actual view */
    void slotFileNew();
    /** open a document */
    void slotFileOpen();
    /** open a document */
    void slotDocOpen();
    void LadeDoc(QString fileName);
    /** save a document */
    void slotFileSave();
    /** save a document under a different filename*/
    void slotFileSaveAs();
    /** Sichert den Text eines Elements */
    void SaveText();
    /** close the actual file */
    void slotFileClose();
    /** print the actual file */
    void slotFilePrint();
    /** exits the application */
    void slotFileQuit();
    /** put the marked text/object into the clipboard and remove
     * it from the document */
    void slotEditCut();
    /** put the marked text/object into the clipboard*/
    void slotEditCopy();
    /** paste the clipboard into the document*/
    void slotEditPaste();
    void EnableTxEdit();
    void DisableTxEdit();
    void SelectAll();
    void DeleteText();

    /** shows an about dlg*/
    void slotHelpAbout();
    void slotHelpAboutQt();
  /** Erzeugt eine neue Seite */
  void slotNewPage();
  /** Lscht die aktuelle Seite */
  void DeletePage();
  /** Ansicht absolut zoomen */
	void slotZoomAbs(float z);
	/** Ansicht ganzes Blatt) */
	void slotZoomFit();
  /** Ansicht 20 % */
	void slotZoom20();
  /** Ansicht 50 % */
	void slotZoom50();
  /** Ansicht 75 % */
	void slotZoom75();
	/** Ansicht 100 % */
	void slotZoom100();
	/** Ansicht 200 % */
	void slotZoom200();
	/** Schaltet Rnder ein/aus */
	void ToggleMarks();
	/** Schaltet Werkzeuge ein/aus */
	void ToggleTools();
	/** Schaltet Mapalette ein/aus */
	void ToggleMpal();
	/** Schaltet bersichtspalette ein/aus*/
	void ToggleTpal();
	/** Schaltet Farbpalette ein/aus */
	void ToggleCpal();
	/** Schaltet Bilder ein/aus */
	void TogglePics();
	/** Schaltet Raster ein/aus */
	void ToggleRaster();
	/** Schaltet Rasterbenutzung ein/aus */
	void ToggleURaster();
	/** Schaltet Rahmenbearbeitung ein/aus */
	void ToggleFrameEdit();
	/** Switch AppMode */
	void setAppMode(int mode);
	/** Neues Dokument erzeugt */
	void HaveNewDoc();
	/** Element ausgewhlt */
	void HaveNewSel(int Nr);
	/** Dokument ist gendert worden */
	void slotDocCh();
	/** Bewegung in der X-Achse */
	void setItemXp();
	/** nderung des Zeilenabstandes */
	void setItemLspS(int w);
	/** Liniennderung */
	void setItemLineWidth(int w);
	/** Lokale XY-nderung */
	void setItemLocalXY(float x, float y);
	/** Lokale Sc-nderung */
	void setItemLocalSc(float x, float y);
	/** Spiegelt das Bild horizontal */
	void setImageH();
	/** Spiegelt das Bild vertikal */
	void setImageV();
	/** Setzt die Linienart */
	void setLineStipple(PenStyle w);
	/** Setzt die Farbe */
	void setItemFarbe(int id);
	/** Setzt die Abstufung */
	void setItemShade(int id);
	/** Setzt den Font */
	void setItemFont(int id);
	/** Korrigiert das FontMenu */
	void AdjustFontMenu(QString nf);
	void SetNewFont(QString nf);
	/** Setz die Zeichensatzgre */
	void setItemFSize(int id);
	void setFSizeMenu(int size);
	void setFontSize(int size);
	/** Farbeditor */
	void slotEditColors();
	/** Setzt den Pen-Tonwert */
	void setPenShade(int sh);
	/** Setzt den Brush-Tonwert */
	void setBrushShade(int sh);
	/** Setzt die Pen-Farbe */
	void setPenFarbe(QString farbe);
	/** Setzt die Brush-Farbe */
	void setBrushFarbe(QString farbe);
	void setCSMenu(QString f, QString l, int fs, int ls);
	/** Fragt nach den Farben */
	void GetBrushPen();
	/** Erzeugt einen rechteckigen Rahmen */
	void RectFrame();
	/** Erzeugt einen ovalen Rahmen */
	void OvalFrame();
	/** Lscht ein Element */
	void DeleteObjekt();
	/** Setzt das Element in den Hintergrund */
	void Objekt2Back();
	/** Setzt das Element in den Vordergrund */
	void Objekt2Front();
	/** Bewegt das Element eine Ebene nach oben */
	void ObjektRaise();
	/** Bewegt das Element nach unten */
	void ObjektLower();
	/** Dupliziert das Element */
	void ObjektDup();
	/** Dupliziert das Element mehrfach*/
	void ObjektDupM();
	/** Setzt die Infos fr das Dokument */
	void InfoDoc();
	/** Reformatiert das Dokument */
	void SetupDoc();
	/** Richtet Objekte aus */
	void ObjektAlign();
	void DoAlign(bool xa, bool ya, bool Vth, bool Vtv, float xdp, float ydp, int xart, int yart);
	void GetAllFonts();
	void BuildFontMenu();
	void slotFontOrg();
	void slotPrefsOrg();
	void SelectFromOutl(int Page, int Item);
	void SelectFromOutlS(int Page);
	void SaveAsEps();
	void SaveAsPDF();
  /** No descriptions
  void slotTest();
  void slotTest2();
  void slotTest3();
  void slotTest4();      */

signals:
	void TextISize(int);
	void TextIFont(QString);
	
  private:

    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    ScribusView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    ScribusDoc *doc;
  
    /** file_menu contains all items of the menubar entry "File" */
    QPopupMenu *fileMenu;
    /** edit_menu contains all items of the menubar entry "Edit" */
    QPopupMenu *editMenu;
    /** StilMenu enthlt das Stilemen */
    QPopupMenu *StilMenu;
    /** ObjMenu enthlt das Objektemen */
    QPopupMenu *ObjMenu;
    /** pageMenu enthlt das Seitenmen */
    QPopupMenu *pageMenu;
    /** view_menu contains all items of the menubar entry "View" */
    QPopupMenu *viewMenu;
    /** ColorMenu enthlt die Farben des Dokuments */
    QPopupMenu *ColorMenu;
    /** SizeTMenu enthlt die Schriftgren */
    QPopupMenu *SizeTMenu;
    /** ShadeMenu enthlt die Tonwerte */
    QPopupMenu *ShadeMenu;
    /** ShapeMenu enthlt die Rahmenformen */
    QPopupMenu *ShapeMenu;
    /** FontMenu enthlt die Fonts */
    QPopupMenu *FontMenu;
    QPopupMenu *FStyleMenu;
    int ShapeEdit;
    int ShapeM;
    int DistM;
    int pgmd;
    int pgmm;
    int Stm;
    int Obm;
    int Markers;
    int Bilder;
    int Ras;
    int uRas;
    int viewTools;
    int viewMpal;
    int viewCpal;
    int viewTpal;
    int fid1;
    int fid2;
    int fid3;
    int fid4;
    int fid5;
    int fid6;
    int fid7;
    int fid8;
    int fid9;
    int fid10;
    int edid1;
    int edid2;
    int edid3;
    int edid4;
    int edid5;
//		QPopupMenu *testMenu;
    QPopupMenu *helpMenu;
    bool HaveDoc;
    float mm2pts(int mm);
    float pts2mm(float pts);
    Tools *werkz;
    Mpalette *Mpal;
    Cpalette *Cpal;
    Tree *Tpal;
    bool BufferSet;
    bool BufferSet2;
    QIntDict<QString> FontID;
    bool Wsh;
    bool Msh;
    bool Csh;
    bool Tsh;
    int HaveGS;
    int HaveConvert;
    int HavePFB;
    bool PDFavailable;
    int DocNr;
};
#endif 

