/***************************************************************************
                          scribusview.cpp  -  description
                             -------------------
    begin                : Fre Apr  6 21:47:55 CEST 2001
    copyright            : (C) 2001 by Franz Schmid
    email                : Franz.Schmid@altmuehlnet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "scribusview.h"
#include "page.h"
#include "pageback.h"
#include <qcolor.h>
#include <qfont.h>
#include <qpixmap.h>
#include <qpointarray.h>
#include <qstringlist.h>
#include "icons.h"

ScribusView::ScribusView(QWidget *parent, ScribusDoc *doc) : QScrollView(parent)
{
	Ready = FALSE;
	Doc = doc;
	enableClipper(TRUE);
	setResizePolicy(Manual);
	Doc->PageC = 0;
	setHScrollBarMode(QScrollView::AlwaysOn);
	setVScrollBarMode(QScrollView::AlwaysOn);
	setMargins(25, 25, 0, 0);
	QFont fo = QFont(font());
	fo.setPointSize(10);
	LE = new QLineEdit(this);
	LE->setFont(fo);
	LE->setText("100.00 %");
	SB1 = new QPushButton(this);
	SB1->setPixmap(QPixmap(Klein));
	SB2 = new QPushButton(this);
	SB2->setPixmap(QPixmap(Gross));
	LA = new QPushButton(this);
	LA->setFont(fo);
	Seitmen = new QPopupMenu(this);
	Seitmen->setFont(fo);
	LA->setText(tr("Page")+" 1");
	LA->setPopup(Seitmen);
	HR = new Hruler(this, Doc);
	VR = new Vruler(this, Doc);
	Pages.clear();
	Ready = TRUE;
	connect(SB1, SIGNAL(clicked()), this, SLOT(slotZoomOut()));
	connect(SB2, SIGNAL(clicked()), this, SLOT(slotZoomIn()));
	connect(LE, SIGNAL(returnPressed()), this, SLOT(Zval()));
	connect(Seitmen, SIGNAL(activated(int)), this, SLOT(GotoPa(int)));
	connect(this, SIGNAL(contentsMoving(int, int)), this, SLOT(setRulerPos(int, int)));
}

ScribusView::~ScribusView()
{
}

/** Waagrechter Scrollbalken */
void ScribusView::setHBarGeometry(QScrollBar &bar, int x, int y, int w, int h)
{
	bar.setGeometry(x+160, y, w-160, h);
	if (Ready)
		{
		LE->setGeometry(x, y, 60, h);
		SB1->setGeometry(x+60, y, 15, h);
		SB2->setGeometry(x+75, y, 15, h);
		LA->setGeometry(x+90, y, 70, h);
		HR->setGeometry(25, 1, w-24, 25);
		}
}

/** Senkrechter Scrollbalken */
void ScribusView::setVBarGeometry(QScrollBar &bar, int x, int y, int w, int h)
{
	bar.setGeometry(x, y, w, h);
	if (Ready)
		{
		VR->setGeometry(1, 25, 25, h-24);
		}
}
void ScribusView::setRulerPos(int x, int y)
{
	HR->offs = x-int(10*Doc->Scale)-2;
	HR->repX = FALSE;
	HR->repaint();
	VR->offs = y-int(10*Doc->Scale)-2;
	VR->repY = FALSE;
	VR->repaint();
}

void ScribusView::Zval()
{
	QStringList w = QStringList::split(" ", LE->text());
	bool ok = FALSE;
	QStringList::Iterator it = w.begin();
	float s = (*it).toFloat(&ok);
	if (ok)
		{
		Doc->Scale = s/100;
		slotDoZoom();
		}
}

/** Fgt eine Seite hinzu */
Page* ScribusView::addPage()
{
	float s = Doc->Scale;
	QWidget* feh = new PageBack(viewport());
	feh->resize(int((Doc->PageB+5)*s), int((Doc->PageH+5)*s));
	Page* fe = new Page(feh, 0, 0, int(Doc->PageB*s), int(Doc->PageH*s), Doc, this);
	Doc->ActPage = fe;
	if (Doc->PageFP)
		{
		if (Doc->PageC % 2 == 0)
			{
			fe->Margins.Right = Doc->PageM.Right;
			fe->Margins.Left = Doc->PageM.Left;
			}
		else
			{
			fe->Margins.Left = Doc->PageM.Right;
			fe->Margins.Right = Doc->PageM.Left;
			}
		}
	else
		{
		fe->Margins.Right = Doc->PageM.Right;
		fe->Margins.Left = Doc->PageM.Left;
		}
	fe->Margins.Top = Doc->PageM.Top;
	fe->Margins.Bottom = Doc->PageM.Bottom;
	fe->PageNr = Doc->PageC;
	if ((Doc->PageAT) && (!Doc->loading))
		{
		if (Doc->PageSp > 1)
			{
			int a;
			float bsp = (Doc->PageB-fe->Margins.Right-fe->Margins.Left-(Doc->PageSp-1)*Doc->PageSpa)/Doc->PageSp;
			float anf = fe->Margins.Left;
			for (a = 0; a < Doc->PageSp; a++)
				{
				fe->PaintText(anf,
											fe->Margins.Top,
											bsp,
											Doc->PageH-fe->Margins.Bottom-fe->Margins.Top,
											1, Doc->Dpen);
				anf += bsp + Doc->PageSpa;
				}
			}
		else
			{
			fe->PaintText(fe->Margins.Left,
										fe->Margins.Top,
										Doc->PageB-fe->Margins.Right-fe->Margins.Left,
										Doc->PageH-fe->Margins.Bottom-fe->Margins.Top,
										1, Doc->Dpen);
				}
		}
	addChild(feh, int(10*s), int(Doc->PageC*((Doc->PageH+30)*s)+10*s));
	resizeContents(int(Doc->PageB*s)+30,	int((Doc->PageH+30)*s)*Doc->PageC+int((Doc->PageH+5)*s)+30);
	feh->show();
	Doc->PageC++;
	Pages.append(fe);
	PaMenu();
	setContentsPos(0, childY(feh)-10*Doc->Scale);
	setRulerPos(contentsX(), contentsY());
	LA->setText(tr("Page")+" "+QString::number(Doc->PageC));
	fe->setMouseTracking(TRUE);
	connect(fe, SIGNAL(Hrule(int)), HR, SLOT(Draw(int)));
	connect(fe, SIGNAL(Vrule(int)), VR, SLOT(Draw(int)));
	connect(fe, SIGNAL(PgCh(int)), this, SLOT(setMenTxt(int)));
	connect(fe, SIGNAL(ZoomIn()), this, SLOT(slotZoomIn()));
	connect(fe, SIGNAL(ZoomOut()), this, SLOT(slotZoomOut()));
	return fe;	
}

/** Lscht eine Seite */
void ScribusView::delPage(int Nr)
{
	if (Doc->PageC == 1)
		{ return; }
	if (Doc->PageC < Nr-1)
		{ return; }
	removeChild(Pages.at(Nr)->parentWidget());
	delete Pages.at(Nr)->parentWidget();
	Pages.remove(Nr);
	Doc->PageC -= 1;
	Doc->ActPage = Pages.at(0); 	
	uint a;
	Page* Seite;
	QWidget* PSeite = Doc->ActPage->parentWidget();
	if (Pages.count() != 0)
		{
		for (a=0; a < Pages.count(); a++)
			{
			Seite = Pages.at(a);
			PSeite = Seite->parentWidget();
			Seite->PageNr = a;
			if (Doc->PageFP)
				{	
				if (a % 2 == 0)
					{
					Seite->Margins.Right = Doc->PageM.Right;
					Seite->Margins.Left = Doc->PageM.Left;
					}
				else
					{
					Seite->Margins.Left = Doc->PageM.Right;
					Seite->Margins.Right = Doc->PageM.Left;
					}
				}
			else
				{
				Seite->Margins.Right = Doc->PageM.Right;
				Seite->Margins.Left = Doc->PageM.Left;
				}
			addChild(PSeite, int(10*Doc->Scale), int(a*(PSeite->height()+25*Doc->Scale)+10*Doc->Scale));
			}
		resizeContents(int(PSeite->width()+30*Doc->Scale), int(Doc->PageC * (PSeite->height()+25*Doc->Scale)+30));
		setContentsPos(0, childY(Doc->ActPage->parentWidget())-int(10*Doc->Scale));
		setRulerPos(0-int(10*Doc->Scale), contentsY());
		PaMenu();
		DrawNew();
		LA->setText(tr("Page")+" 1");
		}	
}

void ScribusView::reformPages()
{	
	uint a;
	Page* Seite;
 	for (a=0; a < Pages.count(); a++)
 		{
 		Seite = Pages.at(a);
		if (Doc->PageFP)
			{	
 			if (a % 2 == 0)
 				{
 				Seite->Margins.Right = Doc->PageM.Right;
 				Seite->Margins.Left = Doc->PageM.Left;
 				}
 			else
 				{
 				Seite->Margins.Left = Doc->PageM.Right;
 				Seite->Margins.Right = Doc->PageM.Left;
 				}
 			}
		else
			{
			Seite->Margins.Right = Doc->PageM.Right;
			Seite->Margins.Left = Doc->PageM.Left;
			}
		Seite->Margins.Top = Doc->PageM.Top;
		Seite->Margins.Bottom = Doc->PageM.Bottom;
		}
 	DrawNew();
}	

void ScribusView::setMenTxt(int Seite)
{
	LA->setText(tr("Page")+" "+QString::number(Seite+1));
}

/** Fhrt die Vergrerung/Verkleinerung aus */
void ScribusView::slotDoZoom()
{ 	
	uint a;
	Page* Seite;
	QWidget* PSeite = Doc->ActPage->parentWidget();
	if (Pages.count() != 0)
		{
		for (a=0; a < Pages.count(); a++)
			{
			Seite = Pages.at(a);
			PSeite = Seite->parentWidget();
			Seite->resize(int(Doc->PageB*Doc->Scale), int(Doc->PageH*Doc->Scale));
			PSeite->resize(int((Doc->PageB+5)*Doc->Scale), int((Doc->PageH+5)*Doc->Scale));
			addChild(PSeite, int(10*Doc->Scale), int(a*(PSeite->height()+25*Doc->Scale)+10*Doc->Scale));
			}
		resizeContents(int(PSeite->width()+30*Doc->Scale), int(Doc->PageC * (PSeite->height()+25*Doc->Scale)+30));
		setContentsPos(0, childY(Doc->ActPage->parentWidget())-int(10*Doc->Scale));
		LE->setText(QString::number(double(Doc->Scale*100), 'f', 2)+" %");
		setRulerPos(contentsX(), contentsY());
		DrawNew();
		}
}

void ScribusView::PaMenu()
{
	uint a;
	Seitmen->clear();
	if (Pages.count() != 0)
		{
		for (a=0; a < Pages.count(); a++)
			{
			Seitmen->insertItem(tr("Page")+" "+QString::number(a+1));
			}
		}
}

void ScribusView::GotoPa(int Seite)
{
	int d = Seitmen->indexOf(Seite);
	GotoPage(d);
}

void ScribusView::GotoPage(int Seite)
{
	Doc->ActPage = Pages.at(Seite);
	setContentsPos(0, childY(Doc->ActPage->parentWidget())-10*Doc->Scale);
	LA->setText(tr("Page")+" "+QString::number(Seite+1));
}

/** Vergrert die Ansicht */
void ScribusView::slotZoomIn()
{
	Doc->Scale *= 2;
	if (Doc->Scale > 32)
		{
		Doc->Scale = 32;
		}
	slotDoZoom();
}

/** Verkleinert die Ansicht */
void ScribusView::slotZoomOut()
{
	Doc->Scale /= 2;
	slotDoZoom();
}

void ScribusView::DrawNew()
{ 	
	uint a;
	if (Pages.count() != 0)
		{
		for (a=0; a < Pages.count(); a++)
			{
			Pages.at(a)->repaint();
			}
		}
}

QString ScribusView::CreatePS(PSLib *p, uint von, uint bis, int step)
{
	uint a, b, d;
	int h, s, v;
	QColor tmp;
	PageItem *c;
	struct Pti *hl;
	QPoint np;
	bool needsStroke = false;
	p->PS_set_Info("Author", Doc->DocAutor);
	p->PS_set_Info("Title", Doc->DocTitel);
//	for (a = von; a < QMIN(bis, Pages.count()); a++)
	a = von;
	while (a != bis)
		{
		p->PS_begin_page(Doc->PageB, Doc->PageH);
		for (b = 0; b < Pages.at(a)->Items.count(); b++)
			{
			c = Pages.at(a)->Items.at(b);
			if (c->isPrintable)
				{
				p->PS_save();
				Doc->PageColors[c->Pcolor].hsv(&h, &s, &v);
				tmp.setHsv(h, s * c->Shade / 100, v);
				tmp.rgb(&h, &s, &v);
				p->PS_setrgbcolor_fill(h / 255.0, s / 255.0, v / 255.0);
				Doc->PageColors[c->Pcolor2].hsv(&h, &s, &v);
				tmp.setHsv(h, s * c->Shade2 / 100, v);
				tmp.rgb(&h, &s, &v);
				p->PS_setrgbcolor_stroke(h / 255.0, s / 255.0, v / 255.0);
				p->PS_setlinewidth(QMAX(c->Pwidth, 1));
				p->PS_setdash(c->PLineArt);
				p->PS_translate(c->Xpos, Doc->PageH - c->Ypos);
				if (c->Rot != 0)
					{
					p->PS_rotate(-c->Rot);
					}
				switch (c->PType)
					{
					case 1:
						p->PS_scale(1, c->Height*1.0 / c->Width);
						p->PS_circle(c->Width/2, (-c->Height / 2) / (c->Height * 1.0 / c->Width), c->Width / 2);
						needsStroke = true;
						break;
					case 2:
						if ((c->flippedH % 2) != 0)
							{
							p->PS_translate(c->Width, 0);
							p->PS_scale(-1, 1);
							}
						if ((c->flippedV % 2) != 0)
							{
							p->PS_translate(0, -c->Height);
							p->PS_scale(1, -1);
							}
						if (!c->Transparent)
							{
							d = 0;
							np = c->Clip.point(d);
							p->PS_moveto(np.x(), -np.y());
							for (d = 1; d < c->Clip.count(); d++)
								{
								np = c->Clip.point(d);
								p->PS_lineto(np.x(), -np.y());
								}
							p->PS_closepath();
							p->PS_fill();
							}
						d = 0;
						np = c->Clip.point(d);
						p->PS_moveto(np.x(), -np.y());
						for (d = 1; d < c->Clip.count(); d++)
							{
							np = c->Clip.point(d);
							p->PS_lineto(np.x(), -np.y());
							}
						p->PS_closepath();
						p->PS_clip();
						p->PS_translate(0, -c->BBoxH*c->LocalScY);
						p->PS_image(-c->BBoxX+c->LocalX, -c->LocalY, c->Pfile, c->LocalScX, c->LocalScY);						
						needsStroke = false;
						break;
					case 3:
						p->PS_rect(0, 0, c->Width, -c->Height);
						needsStroke = true;
						break;
					case 4:
						if (c->isBookmark)
							{
							QString bm = "";
							for (d = 0; d < c->Ptext.count(); d++)
								{
								bm += QChar(c->Ptext.at(d)->ch);
								}
							p->PDF_Bookmark(bm, a+1);
							}
						if (!c->Transparent)
							{
							d = 0;
							np = c->Clip.point(d);
							p->PS_moveto(np.x(), -np.y());
							for (d = 1; d < c->Clip.count(); d++)
								{
								np = c->Clip.point(d);
								p->PS_lineto(np.x(), -np.y());
								}
							p->PS_closepath();
							p->PS_fill();
							}
						for (d = 0; d < c->Ptext.count(); d++)
							{
							hl = c->Ptext.at(d);
							if ((hl->ch == 13) || (hl->ch == 10) || (hl->ch == 32))
								continue;
							p->PS_selectfont(hl->cfont, hl->csize);
							Doc->PageColors[hl->ccolor].hsv(&h, &s, &v);
							tmp.setHsv(h, s * hl->cshade / 100, v);
							tmp.rgb(&h, &s, &v);
							p->PS_setrgbcolor_stroke(h / 255.0, s / 255.0, v / 255.0);
							p->PS_show_xy(hl->ch, hl->xp, -hl->yp);
							}
						needsStroke = false;
						break;
					case 5:
						p->PS_moveto(0, 0);
						p->PS_lineto(c->Width, -c->Height);
						p->PS_stroke();
						needsStroke = false;
						break;
					}
				if (needsStroke)
					{
					if (c->Transparent)
						{
						p->PS_stroke();
						}
					else
						{
						if (c->Pwidth == 0)
							{
							p->PS_fill();
							}
						else
							{
							p->PS_fill_stroke();
							}
						}
					}
				p->PS_restore();
				}
			}
		p->PS_end_page();
		a += step;
		}
	return p->PS_close();
}
