/***************************************************************************
                          serializer.cpp  -  description
                             -------------------
    begin                : Sat May 5 2001
    copyright            : (C) 2001 by Franz Schmid
    email                : Franz.Schmid@altmuehlnet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "serializer.h"
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qpointarray.h>
Serializer::Serializer(QString name)
{
	Filename = name;
	Objekt = "";
	Info = "";
	Farben = "";
	Seiten = "";
	Schriften = "";
	Count = 0;
}
Serializer::~Serializer()
{
}

QString Serializer::GetObjekt()
{
	return Objekt;
}

void Serializer::PutText(PageItem *Item)
{
  uint a;
	QList<Pti> y = Item->Ptext;
  for (a=0; a<y.count(); a++)
  	{
    Objekt += y.at(a)->ch;
    }
}

void Serializer::GetText(PageItem *Item)
{
	struct Pti *hg;
	uint a;
	Item->Ptext.clear();
	for (a=0; a<Objekt.length(); a++)
		{
		hg = new Pti;
		hg->ch = Objekt.at(a).unicode();
		if (hg->ch == 10) { hg->ch = 13; }
		hg->cfont = Item->IFont;
		hg->csize = Item->ISize;
		hg->ccolor = Item->Pcolor2;
		hg->cextra = 0;
		hg->cshade = Item->Shade2;
		hg->cselect = false;
		Item->Ptext.append(hg);
		}
}

void Serializer::PutColors(CListe farben)
{
	CListe::Iterator it;
	QColor tmp;
	QString cc;
	int r, g, b;
	Objekt += "Colors\n";
	for (it = farben.begin(); it != farben.end(); ++it)
		{
		tmp = farben[it.key()];
		tmp.rgb(&r, &g, &b);
		Objekt += it.key()+" "+cc.setNum(r)+" "+cc.setNum(g)+" "+cc.setNum(b)+"\n";
		}
	Objekt += "Part\n";
}

CListe Serializer::GetColors()
{
	CListe farben;
	QString key;
	int r, g, b;
	QString tmp;
	QTextStream t(&Farben, IO_ReadOnly);
	while (!t.atEnd())
		{
		tmp = t.readLine();
		QTextStream f(&tmp, IO_ReadOnly);
		f >> key >> r >> g >> b;
		farben.insert(key, QColor(r, g, b));
		}
	return farben;
}

void Serializer::PutInfo(ScribusDoc *doc)
{
	Objekt += "Info\n";
	Objekt += doc->DocTitel+"\n";
	Objekt += doc->DocAutor+"\n";
	Objekt += doc->DocComments+"\n";
	Objekt += "Part\n";
}

void Serializer::GetInfo(ScribusDoc *doc)
{
	QTextStream t(&Info, IO_ReadOnly);
	doc->DocTitel = t.readLine();
	doc->DocAutor = t.readLine();
	doc->DocComments = t.read();
	doc->DocComments.remove(doc->DocComments.length()-1, 1);
}

void Serializer::PutValues(ScribusDoc *doc)
{
	QString cc;
	QTextStream t(&cc, IO_WriteOnly);
	Objekt += "Values\n";
	t << doc->PageC << endl;
	t << doc->PageB << " ";
	t << doc->PageH << endl;
	t << doc->PageM.Top << " ";
	t << doc->PageM.Left << " ";
	t << doc->PageM.Right << " ";
	t << doc->PageM.Bottom << endl;
	t << doc->PageSp << " ";
	t << doc->PageSpa << endl;
	t << int(doc->PageAT) << " ";
	t << int(doc->PageFP) << endl;
	t << doc->Dwidth << " ";
	t << doc->Dshade << " ";
	t << doc->Dshade2 << endl;
	t << doc->Dpen << " ";
	t << doc->Dbrush << " ";
	t << int(doc->DLineArt) << endl;
	Objekt += cc + "Part\n";
}

void Serializer::PutFonts(ScribusDoc *doc)
{
	Objekt += "Fonts\n";
  QMap<QString,QFont>::Iterator it;
	for (it = doc->UsedFonts.begin(); it != doc->UsedFonts.end(); ++it)
		{
		Objekt += it.key()+"\n";
		}	
	Objekt += "Part\n";
}

QString Serializer::GetFonts()
{
	return Schriften;
}

void Serializer::GetValues(ScribusDoc *doc)
{
	QTextStream t(&Werte, IO_ReadOnly);
	int tmp;
	t >> Count;
	t >> doc->PageB;
	t >> doc->PageH;
	t >> doc->PageM.Top;
	t >> doc->PageM.Left;
	t >> doc->PageM.Right;
	t >> doc->PageM.Bottom;
	t >> doc->PageSp;
	t >> doc->PageSpa;
	t >> tmp;
	doc->PageAT = bool(tmp);
	t >> tmp;
	doc->PageFP = bool(tmp);
	t >> doc->Dwidth;
	t >> doc->Dshade;
	t >> doc->Dshade2;
	t >> doc->Dpen;
	t >> doc->Dbrush;
	t >> tmp;
	doc->DLineArt = Qt::PenStyle(tmp);
}

void Serializer::PutPage(ScribusView *view)
{
	uint a, b, c;
	int x, y;
	struct CLBuf Buffer;
	Page *pg;
	for (c=0; c<view->Pages.count(); c++)
		{
		Objekt += "Page\n";
		pg = view->Pages.at(c);
		QString cc = "";
		QTextStream t(&cc, IO_WriteOnly);
		for (a=0; a<pg->Items.count(); a++)
			{
			pg->Items.at(a)->CopyIt(&Buffer);
			t << Buffer.PType << endl;
			t << Buffer.Xpos << endl;
			t << Buffer.Ypos << endl;
			t << Buffer.Height << endl;
			t << Buffer.Width << endl;
			t << Buffer.LocalScX << endl;
			t << Buffer.LocalScY << endl;
			t << Buffer.LocalX << endl;
			t << Buffer.LocalY << endl;
			t << Buffer.Extra << endl;
			t << Buffer.Rot << endl;
			t << Buffer.Pwidth << endl;
			t << Buffer.LineSp << endl;
			t << Buffer.Shade << endl;
			t << Buffer.Shade2 << endl;
			t << Buffer.Pcolor << endl;
			t << Buffer.Pcolor2 << endl;
			if (Buffer.Pfile == "")
				{
				t << "-1" << endl;
				}
			else
				{
				t << Buffer.Pfile << endl;
				}
			if (Buffer.Ptext == "")
				{
				t << "End" << endl;
				}
			else
				{
				t << Buffer.Ptext << "End" << endl;
				}
			t << int(Buffer.PLineArt) << endl;
			t << int(Buffer.PicArt) << endl;
			t << int(Buffer.Transparent) << endl;
			t << Buffer.flippedH << endl;
			t << Buffer.flippedV << endl;
			t << Buffer.BBoxX << endl;
			t << Buffer.BBoxH << endl;
			t << int(Buffer.isBookmark) << endl;
			t << int(Buffer.isPrintable) << endl;
			t << Buffer.Clip.size() << endl;
			for (b=0; b<Buffer.Clip.size(); b++)
				{
				Buffer.Clip.point(b, &x, &y);
				t << x << " " << y << " ";
				}
			t << endl;
			}
		Objekt += cc;
		}
	Objekt += "Part\n";
}

void Serializer::GetPage(ScribusView *view, QMap<QString,QString> ersa)
{
	int a = -1;
	int tmp, x, y;
	uint b, c;
	float tmpf;
	QString tmps, tmps2;
	struct CLBuf Buffer;
	QString cc = "";
	QStringList wt;
	QStringList::Iterator it;
	wt = QStringList::split("Page\n", Seiten);
	for (it = wt.begin(); it != wt.end(); ++it)
		{
		cc = *it;
		a++;
		QTextStream t(&cc, IO_ReadOnly);
		while (!t.atEnd())
			{
			t >> tmp;
			Buffer.PType = tmp;
			t >> tmpf;
			Buffer.Xpos = tmpf;
			t >> tmpf;
			Buffer.Ypos = tmpf;
			t >> tmpf;
			Buffer.Height = tmpf;
			t >> tmpf;
			Buffer.Width = tmpf;
			t >> tmpf;
			Buffer.LocalScX = tmpf;
			t >> tmpf;
			Buffer.LocalScY = tmpf;
			t >> tmpf;
			Buffer.LocalX = tmpf;
			t >> tmpf;
			Buffer.LocalY = tmpf;
			t >> tmp;
			Buffer.Extra = tmp;
			t >> tmpf;
			Buffer.Rot = tmpf;
			t >> tmp;
			Buffer.Pwidth = tmp;
			t >> tmp;
			Buffer.LineSp = tmp;
			t >> tmp;
			Buffer.Shade = tmp;
			t >> tmp;
			Buffer.Shade2 = tmp;
			t >> tmps;
			Buffer.Pcolor = tmps;
			t >> tmps;
			Buffer.Pcolor2 = tmps;
			t >> tmps;
			Buffer.Pfile = "";
			if (tmps != "-1")
				{
				Buffer.Pfile = tmps;
				}
			tmps2 = "";
			if (Buffer.PType == 4)
				{
				do
					{
					tmps = t.readLine();
					if (tmps == "End") { break; }
					if (tmps != "")
						{
						if (ersa.count() != 0)
							{
							QMap<QString,QString>::Iterator itx;
							for (itx = ersa.begin(); itx != ersa.end(); ++itx)
								{
								tmps = tmps.replace(QRegExp(itx.key()), itx.data());
								}
							}
						tmps2 += tmps+"\n";
						}
					}
				while (tmps != "End");
				Buffer.Ptext = tmps2;
				}
			else
				{
				tmps = t.readLine();
				tmps = t.readLine();
				Buffer.Ptext = "";
				}
			t >> tmp;
			Buffer.PLineArt = Qt::PenStyle(tmp);
			t >> tmp;
			Buffer.PicArt = bool(tmp);
			t >> tmp;
			Buffer.Transparent = bool(tmp);
			t >> tmp;
			Buffer.flippedH = tmp;
			t >> tmp;
			Buffer.flippedV = tmp;
			t >> tmp;
			Buffer.BBoxX = tmp;
			t >> tmp;
			Buffer.BBoxH = tmp;
			t >> tmp;
			Buffer.isBookmark = bool(tmp);
			t >> tmp;
			Buffer.isPrintable = bool(tmp);
			t >> b;
			Buffer.Clip.resize(b);
			tmps = t.readLine();
			tmps = t.readLine();
			QTextStream f(&tmps, IO_ReadOnly);
			for (c=0; c<b; c++)
				{
				f >> x;
				f >> y;
				Buffer.Clip.setPoint(c, x, y);
				}
			view->Pages.at(a)->PasteItem(&Buffer, true);
			}
	  }
}

void Serializer::PutDoc(ScribusDoc *doc)
{
	PutValues(doc);
	PutInfo(doc);
	PutFonts(doc);
	PutColors(doc->PageColors);
}

void Serializer::GetDoc()
{
	QStringList wt;
	QStringList::Iterator it;
	wt = QStringList::split("Part\n", Objekt);
	for (it = wt.begin(); it != wt.end(); ++it)
		{
		if ((*it).startsWith("Values"))
			{
			Werte = *it;
			Werte.remove(0, 7);
			}
		if ((*it).startsWith("Info"))
			{
			Info = *it;
			Info.remove(0, 5);
			}
		if ((*it).startsWith("Colors"))
			{
			Farben = *it;
			Farben.remove(0, 7);
			}
		if ((*it).startsWith("Page"))
			{
			Seiten = *it;
			Seiten.remove(0, 5);
			}
		if ((*it).startsWith("Fonts"))
			{
			Schriften = *it;
			Schriften.remove(0, 6);
			}
		}
}

bool Serializer::Write()
{
	QFile f(Filename);
	bool ret = false;
	if (f.open(IO_WriteOnly))
		{
		QTextStream t(&f);
		t.writeRawBytes(Objekt, Objekt.length());
		f.close();
		ret = true;
		}
	else
		{
		ret = false;
		}
	return ret;
}

bool Serializer::Read()
{
	QFile f(Filename);
	bool ret = false;
	if (f.open(IO_ReadOnly))
		{
		QTextStream t(&f);
		Objekt = t.read();
		f.close();
		ret = true;
		}
	else
		{
		ret = false;
		}
	return ret;
}
